proc printStack {} {
    set level [info level]
    for {set i 1} {$i < $level} {incr i} {
	puts "Level $i: [info level $i]"
    }
}

#these have to go out here because they're used by powSetupColormap which is
#called before powInit
set powMinColorcells 60
#   Use pseudocolor images by default 
set powPseudoImages 1
#   Also need this so powToplevel can get going in safe mode
set powbg tan
#   By default, assume we're not running safe (need to make a powSafeInit entry
#   point to do this eventually
set powFrameForTop 0

trace variable powWCS w powWCSCallback

proc powWCSCallback {array element op} {
    global powWCS
    if {$powWCS($element) == ""} return
    if {![catch {image type $element}]} {
	eval [concat powWCSInitImage $element $powWCS($element)];
    } elseif { [lsearch -exact [powListCurves] $element]!=-1 } {
	eval [concat powWCSInitCurve $element $powWCS($element)];
    }
}

proc powInit { {powXColormapWindow none} {powContainerPath none} {powgui 1}} {
#This procedure creates the .pow window and sets up all the bindings
#  powXColormapWindow -  a Tk path specifying a window to use to find the
#                        X colormap to use.
#  powContainer - what window to "use" to put POW in
#  powgui - Do you want the POW user GUI header (usually yes, unless you're
#           embedding POW in some other application)
    global currgn currimg mag powfont Private_Colormap env
    global cmap cmap_inv powLut powResizeMain powcursor
    global powCurveParam
    global powbg powScopeWidth powScopeHeight powPlotParam powMinWidth
    global powMinHeight powMaxHeight powMaxWidth powFirstTimeThroughFlag
    global powShowHandlesFlag powHeaderHeight powTrackText 
    global powGraphCoordsTracker powImagePixelTracker  powImageValueTracker
    global powHelpTopics curr_img powFirstPixel
    global powSqueeze powSlide powPseudoImages powMinColorcells
    global  currimggn Pow_Allocated powSixties powNumTicks
    global powGrid powGridColor powGridDash
    global powGUI showlinks powContainer powFrameForTop

    set powGUI $powgui
    set powContainer $powContainerPath
    if {($powXColormapWindow == "safe" || $powFrameForTop == 1) && \
	    $powContainer == "none"} {
	set powContainer "."
    }

    if {!$powGUI} {
	set powHeaderHeight 0
	set powMinWidth 0
    }

    event add <<powExit>> <Control-c>
    set powFirstTimeThroughFlag 1
    set powShowHandlesFlag 1


#    trace variable powPlotParam w debug_trace


#set defaults for options

    if {$powXColormapWindow == "safe"} {
#don't set up a colormap if we're running in a tclet
	set powFrameForTop 1
	if {[winfo visual $powContainer] == "pseudocolor"} {
#	    set powPseudoImages 1
#	    This seems to break the plugin, so for now
	    set powPseudoImages 0
	} else {
	    set powPseudoImages 0
	}
	powToplevel .pow safe
    } else {	
        if {$powXColormapWindow == "none" || $powXColormapWindow == "NULL"} {
	    if {$powContainer != "none" && $powContainer != "NULL"} {
		powSetupColormap .pow 0 [list -use [winfo id $powContainer]]
	    } else {
		powSetupColormap .pow 0 
	    }
	    set powXColormapWindow .pow 
	} else {
	    set visual [winfo visual $powXColormapWindow]
	    set depth [winfo depth $powXColormapWindow]
	    if {![regexp -nocase "pseudocolor" $visual] || $depth != 8} {
		puts stderr "Visual of window $powXColormapWindow is $visual $depth."
		set powPseudoImages 0
	    }
	    
	    if {$powContainer != "none" && $powContainer != "NULL"} {
		powToplevel .pow $powXColormapWindow [list -use [winfo id $powContainer]]
	    } else {
		powToplevel .pow $powXColormapWindow 
	    }
	}



#Check that there's still enough colors for POW to function (i.e. they haven't
#filled up the Colormap since we set it up, this should only happen if 
#they're using the default colormap).
	if {$powPseudoImages} {
	    set free_colors [powTestColormap $powXColormapWindow]
	    if {(($Pow_Allocated != 0) && ($free_colors < 10)) || \
		    ($Pow_Allocated == 0) && ($free_colors < 60) } {
		puts stderr "Colormap full";
		set powPseudoImages 0;
	    }
	}
    }

#powPseudoImages is set by powSetupColormap
    if {!$powPseudoImages} {
#	puts stderr "Pseudocolor images disabled; using Truecolor images."
	powSetupPhotoImages
    }

    
    
    label .pow.testfont 
    .pow.testfont config -font fixed
    set powfont [powFindFont .pow.testfont [list 120 130] ]
    set powbg "tan"
    set cmap spectrum
    set cmap_inv 0
    set powLut log

    set powCurveParam(allOpts,powDef) \
	    [list pDisp pShape pSizeErr pSize pFill lDisp lStyle lWidth lStep Color]
    foreach opt $powCurveParam(allOpts,powDef) \
	    val [list 1 Cross 0 4 No 0 " " 1 0 Black] {
	set powCurveParam($opt,powDef) $val
    }

    set powcursor crosshair
    set powResizeMain 1
    set showlinks 0
    set powScopeWidth 200
    set powScopeHeight 200
    set powMinHeight 500
    set powFirstPixel 1
    set powSixties 1
    set powNumTicks 3
    set powGrid 1
    set powGridColor "White"
    set powGridDash " "

#read user's option file if present
    catch {if [file readable "~/.powrc"] { source "~/.powrc"}}

# test for obsolete variable powCurvetype

    if { [info exists powCurvetype] } {
	if { [string first Points $powCurvetype]==-1 } {
	    set powCurveParam(pDisp,powDef) 0
	} else {
	    set powCurveParam(pDisp,powDef) 1
	}
	if { [string first Line $powCurvetype]==-1 } {
	    set powCurveParam(lDisp,powDef) 0
	} else {
	    set powCurveParam(lDisp,powDef) 1
	}
	unset powCurvetype
    }


    set powSqueeze 0.0
    set powSlide 0.0


#    source $env(TCLTK_LIBRARY)/pow/html_library.tcl
#    tk_setPalette $powbg
    
    set Private_Colormap 4
    
    if {!$powPseudoImages} {
	powSetColorTable
    }

    .pow configure -bg $powbg
    set revisionString {$Revision: 1.234 $}
    regexp {Revision: ([0-9.]+)} $revisionString blob revno
    catch {wm title .pow "POW $revno"}

    if $powGUI {
	frame .pow.scopebuttons -bg $powbg
	button .pow.scopein -bg $powbg -text "Zoom In" \
	    -command "powScopeZoom in"
	button .pow.scopezoom1 -bg $powbg -text "Replot" \
	    -command "powEndROI 1"
	button .pow.scopeout -bg $powbg -text "Zoom Out" \
	    -command "powScopeZoom out"
	canvas .pow.scope -bg $powbg -cursor $powcursor -width $powScopeWidth\
	    -height $powScopeHeight 
	menubutton .pow.colors -text Colors -menu .pow.colors.menu -bg $powbg\
	    -relief raised
	menu .pow.colors.menu -bg $powbg 
	foreach color {gray ct hot cold hls spectrum} {
	    .pow.colors.menu add radiobutton -label $color -variable cmap\
		-value $color -command {
                    if [info exists curr_img] {
			powSetColorTable
			powReditherImages $currimggn $currimg
		    }
		}
	}
	.pow.colors.menu add separator
	.pow.colors.menu add checkbutton -label "Invert Colortable" \
	    -variable cmap_inv -command {
		if [info exists curr_img] {
		    powSetLut $powLut; 
		    powCmapInvert; 
		    powReditherImages $currimggn $currimg;
		}
	    }
	.pow.colors.menu add separator
	.pow.colors.menu add radiobutton -label linear -variable powLut \
	    -value linear -command \
	    {if [info exists curr_img]\
		 {powSetLut $powLut; powReditherImages $currimggn $currimg}}
	.pow.colors.menu add radiobutton -label "square root" -variable powLut\
	    -value sqrt -command \
	    {if [info exists curr_img] \
		 {powSetLut $powLut; powReditherImages $currimggn $currimg}}
	.pow.colors.menu add radiobutton -label logarithmic -variable powLut\
	    -value log -command \
	    {if [info exists curr_img] \
		 {powSetLut $powLut; powReditherImages $currimggn $currimg}}
	.pow.colors.menu add command -label "Rescale Image" \
	    -command {powRescaleBox}
	.pow.colors.menu add separator
	.pow.colors.menu add command -label "Make Colorbar for Current Graph" \
	    -command {powColorbar}
    
	set powHelpTopics(About.html) About
	set powHelpTopics(Overview.html) Overview
	set powHelpTopics(Moving_Graphs.html) "Moving Graphs"
	set powHelpTopics(ROI.html) "Panning/Zooming"
	set powHelpTopics(File.html) "File Menu"
	set powHelpTopics(Options.html) "Options Menu"
	set powHelpTopics(Color.html) "Colors"
	set powHelpTopics(Edit.html) "Edit Menu"
	set powHelpTopics(Regions.html) "Region Files"
	set powHelpTopics(Blinking.html) "Blinking"
	set powHelpTopics(Contours.html) "Contours"
	set powHelpTopics(CurveOptions.html) "Curve Options"
	set powHelpTopics(EditGraphs.html) "Editting Graphs"
	set powHelpTopics(EditObjects.html) "Editting Objects"
    
	menubutton .pow.help -bg $powbg -menu .pow.help.menu -text "Help"\
	    -relief raised
	menu .pow.help.menu -bg $powbg
	foreach topic [lsort [array names powHelpTopics]] {
	    .pow.help.menu add command -label $powHelpTopics($topic) \
		-command "powHelp $topic" \
	    }
	menubutton .pow.edit -text Edit -menu .pow.edit.menu -bg $powbg \
	    -relief raised
	menu .pow.edit.menu -bg $powbg
	menu .pow.edit.menu.unmap -bg $powbg \
	    -postcommand {powGenerateUnmapCascade}
	.pow.edit.menu add cascade -label "Hide Graph" \
	    -menu .pow.edit.menu.unmap
	menu .pow.edit.menu.replot -bg $powbg \
	    -postcommand {powGenerateReplotCascade}  
	.pow.edit.menu add cascade -label "Replot Graph" \
	    -menu .pow.edit.menu.replot
	.pow.edit.menu add command -label "Choose Graph Size" \
	    -command powSetGraphSize
	.pow.edit.menu add command -label "Add Text Label" \
	    -command {powAddTextToGraphDialog}
	.pow.edit.menu add command -label "Edit Graph..." \
	    -command {powEditGraphDlg $currgn}
	menu .pow.edit.menu.merge -bg $powbg -postcommand \
	    {powGenerateMergeCascade}
	.pow.edit.menu add cascade -label "Merge Graphs" \
	    -menu .pow.edit.menu.merge 
	.pow.edit.menu add command -label "Blink Graphs..." \
	    -command {powBlinkGraphDlg}
	.pow.edit.menu add command -label "Blink Images..." \
	    -command {powMovie}
	.pow.edit.menu add command -label "Region Files..." \
	    -command {powRegion}
	.pow.edit.menu add command -label "Make Contour Map..." \
	    -command {powContour}
	menubutton .pow.options -text Options -menu .pow.options.menu \
	    -bg $powbg -relief raised 
	menubutton .pow.file -text File -menu .pow.file.menu \
	    -bg $powbg -relief raised
	menu .pow.file.menu  -bg $powbg
	menu .pow.options.menu -bg $powbg
	.pow.options.menu add command -label "Change Background Color" \
	    -command {set powbgnew [tk_chooseColor -initialcolor $powbg];\
			  if {$powbgnew != ""} {set powbg $powbgnew}; \
			  powChangeBg} 
	.pow.options.menu add command -label "Change Font" \
	    -command {powChangeFontBox} 
	.pow.options.menu add checkbutton -label \
	    "Resize Main Window on Expand" -variable powResizeMain  
	.pow.options.menu add checkbutton -label "Show Linked Axes" \
	    -variable showlinks -command {powShowLinks} 
	menu .pow.options.menu.grid -bg $powbg
	.pow.options.menu add cascade -label "Grid Line Options" \
	    -menu .pow.options.menu.grid
	.pow.options.menu.grid add checkbutton -label "Show Grid Lines" \
	    -variable powGrid -command {powChangeGrid 1}
	.pow.options.menu.grid add separator
	.pow.options.menu.grid add radiobutton -label "White" \
	    -variable powGridColor -value "White" -command {powChangeGrid 0}
	.pow.options.menu.grid add radiobutton -label "Black" \
	    -variable powGridColor -value "Black" -command {powChangeGrid 0}
	.pow.options.menu.grid add radiobutton -label "Blue" \
	    -variable powGridColor -value "Blue" -command {powChangeGrid 0}
	.pow.options.menu.grid add radiobutton -label "Red" \
	    -variable powGridColor -value "Red" -command {powChangeGrid 0}
	.pow.options.menu.grid add separator
	.pow.options.menu.grid add radiobutton -label "Solid" \
	    -variable powGridDash -value " " -command {powChangeGrid 0}
	.pow.options.menu.grid add radiobutton -label "Small Dash" \
	    -variable powGridDash -value 10 -command {powChangeGrid 0}
	.pow.options.menu.grid add radiobutton -label "Large Dash" \
	    -variable powGridDash -value 20 -command {powChangeGrid 0}
	.pow.options.menu.grid add separator
	.pow.options.menu.grid add command -label "Fewer Lines" \
	    -command {if {$powNumTicks>0} {incr powNumTicks -1};\
			  powChangeGrid 1}
	.pow.options.menu.grid add command -label "More Lines" \
	    -command {incr powNumTicks; powChangeGrid 1}
	menu .pow.options.menu.cursor -bg $powbg
	.pow.options.menu add cascade -label "Main Cursor Shape" \
	    -menu .pow.options.menu.cursor
	foreach cursor {crosshair top_left_arrow right_ptr gumby trek} {
	    .pow.options.menu.cursor add radiobutton -label $cursor \
		-variable powcursor -value $cursor \
		-command ".pow.pow configure -cursor $cursor; .pow.scope configure -cursor $cursor;"
	}
	.pow.options.menu add command -label "Curve Options..." \
		-command {powCurveOptions}
	.pow.options.menu add separator
	.pow.options.menu add radiobutton -label "Decimal Axis Labels" \
	    -variable powSixties -value 0  \
	    -command {
		powAdornGraph $currgn .pow.pow; 
		powRedrawTicks $currgn .pow.pow
	    }
	.pow.options.menu add radiobutton -label "Base 60 Axis Labels" \
	    -variable powSixties -value 1 -command {
		powAdornGraph $currgn .pow.pow; 
		powRedrawTicks $currgn .pow.pow
	    }
	.pow.options.menu add command -label "Save Current Settings" \
	    -command {powSaveConfig}

	.pow.file.menu add command -label "Print/Save Plot" \
	    -command {powPrintBox}
	.pow.file.menu add command -label "Close POW" \
	    -command {powExit; set Pow_Done 1} 

	label .pow.currgn -textvariable currgn -background yellow \
	    -relief sunken -foreground black
	set powTrackText(img) "        "
	set powTrackText(rx) "        "
	set powTrackText(ry) "        "
	set powTrackText(imgx) "     "
	set powTrackText(imgy) "     "
	set powTrackText(imgz) "     "
	set powTrackText(zunits) "     "
	powUpdateTrackVars
    
#     trace variable powTrackText w powUpdateTrackVars

	set powTrackerWidth 50
	frame .pow.trackers -bg $powbg -width $powTrackerWidth
	label .pow.graphtrack  -textvariable powGraphCoordsTracker \
	    -background $powbg -relief sunken -anchor w -width $powTrackerWidth
	label .pow.pixeltrack  -textvariable powImagePixelTracker \
	    -background $powbg -relief sunken -anchor w -width $powTrackerWidth
	label .pow.pixvaltrack -textvariable powImageValueTracker \
	    -background $powbg -relief sunken -anchor w -width $powTrackerWidth
    } 
#end powGUI if block

    frame .pow.bottom -bg $powbg 
    canvas .pow.pow -bg $powbg -cursor $powcursor\
	-xscrollcommand ".pow.scrollx set" \
	-yscrollcommand ".pow.scrolly set" -scrollregion [list 0 0 50 50]
    scrollbar .pow.scrolly -command ".pow.pow yview" -orient vertical\
	-bg $powbg
    scrollbar .pow.scrollx -command ".pow.pow xview" -orient horizontal\
	-bg $powbg


    if $powGUI {
	grid configure .pow.file -row 0 -column 0 -sticky w    
	grid configure .pow.options -row 0 -column 1 -sticky w
	grid configure .pow.colors -row 0 -column 2 -sticky w
	grid configure .pow.edit -row 0 -column 3 -sticky w

	grid configure .pow.trackers -row 1 -column 0 -columnspan 7  -sticky w
	grid configure .pow.graphtrack -row 0 -column 0 -sticky "e w" \
	    -padx 5 -in .pow.trackers
	grid configure .pow.pixeltrack -row 1 -column 0 -sticky "e w" \
	    -padx 5 -in .pow.trackers
	grid configure .pow.pixvaltrack -row 2 -column 0 -sticky "e w" \
	    -padx 5 -in .pow.trackers 
	grid configure .pow.currgn -row 0 -column 7 
	grid configure .pow.scope -row 1 -column 7  -sticky "e"
	grid configure .pow.help -sticky e -row 0 -column 8
	grid configure .pow.scopebuttons  -row 1 -column 8 -sticky "n"
	grid configure .pow.scopein  -row 0 -column 0 -sticky "e w" \
	    -in .pow.scopebuttons
	grid configure .pow.scopezoom1 -row 1 -column 0 -sticky "e w" \
	    -in .pow.scopebuttons
	grid configure .pow.scopeout  -row 2 -column 0  -sticky "e w" \
	    -in .pow.scopebuttons
	grid configure .pow.bottom -row 2 -column 0 -columnspan 10 \
	    -sticky "n s e w"
	grid rowconfigure .pow  2  -weight 1
	grid columnconfigure .pow  6 -weight 1
    } else {
	grid configure .pow.bottom -row 0 -column 0 -sticky "n s e w"
	grid rowconfigure .pow  0  -weight 1
	grid columnconfigure .pow  0 -weight 1
    }

    grid configure .pow.pow -sticky "n s e w" -row 0 -column 0 \
	-in .pow.bottom
    grid configure .pow.scrolly -sticky "n s e"  -row 0 -column 1 \
	-in .pow.bottom
    grid configure .pow.scrollx -sticky "n e w" -row 1 -column 0 \
	-in .pow.bottom
    
    
	
    grid rowconfigure .pow.bottom 0 -weight 1
    grid columnconfigure .pow.bottom 0 -weight 1

#put .pow.top above .pow.pow in the stacking order, 
#This should prevent window items on the
#.pow.pow canvas from "bleeding" off the edges of the .pow.pow canvas
    lower .pow.bottom

    update idletasks
	
    if $powGUI {
    
	set powHeaderHeight [lindex [grid bbox .pow 0 2 ] 1]
	
	
	set powMaxWidth [winfo screenwidth .pow]
	set powMaxHeight [expr [winfo screenheight .pow] - $powHeaderHeight]
	set powMinWidth [expr [winfo width .pow.graphtrack] + $powScopeWidth \
			     + [winfo width .pow.scopeout] + 20] 


	bind .pow.pow   <Motion> {set_tracker_info %x %y .pow.pow}
	bind .pow.scope <Motion> {set_tracker_info %x %y .pow.scope}

	bind .pow.pow   <ButtonPress-3>   {powB3 %T ROI {powBeginROI %x %y .pow.pow}}
	bind .pow.pow   <B3-Motion>       {powB3 %T ROI {powDragROI %x %y .pow.pow}}
	bind .pow.pow   <ButtonRelease-3> {powB3 %T ROI {powEndROI 0 .pow.pow}}

	bind .pow.scope <ButtonPress-3>   {powBeginROI %x %y .pow.scope}
	bind .pow.scope <B3-Motion>       {powDragROI %x %y .pow.scope}
	bind .pow.scope <ButtonRelease-3> {powEndROI 0 .pow.scope}

	bind .pow.scope <ButtonPress-1>   {catch {powPanROI %x %y .pow.scope}}
	bind .pow.scope <B1-Motion>       {catch {powPanROI %x %y .pow.scope}}
	bind .pow.scope <ButtonRelease-1> {catch {powEndROI 0 .pow.scope}}

    } else {
	set powMaxWidth [winfo screenwidth .pow]
	set powMaxHeight [expr [winfo screenheight .pow] - $powHeaderHeight]
    }

    if {$powContainer == "none" || $powContainer == "NULL" } {
	catch {wm minsize .pow $powMinWidth [expr $powMinHeight + $powHeaderHeight]}
	catch {wm maxsize .pow $powMaxWidth [expr $powMaxHeight + $powHeaderHeight]}
    } else {
	catch {wm geometry .pow "[winfo geometry $powContainer]"}
    }

    bind .pow.pow <ButtonPress-1> {powProcessClick %x %y B1}
	
    bind .pow.pow <ButtonPress-2> {powProcessClick %x %y B2}

    bind .pow.pow <Double-Button-1> {powProcessClick %x %y B1D}

    bind .pow <Down> {event generate .pow <Motion> -warp yes -x %x \
			  -y [expr %y+1]}
    bind .pow <Up> {event generate .pow <Motion> -warp yes -x %x \
			-y [expr %y-1]}
    bind .pow <Left> {event generate .pow <Motion> -warp yes -x [expr %x - 1] \
			  -y %y}
    bind .pow <Right> {event generate .pow <Motion> -warp yes \
			   -x [expr %x + 1] -y %y}
}

# This function prevents multiple bindings from being executed for B3 events.
# Only the first set of events will be accepted.

proc powB3 { evt cntxt cmd } {
    global powB3state powB3context

    if { ![info exists powB3state] } {
	set powB3state release
	set powB3context none
    }

    # If this is a different event from last time, execute command and remember
    # context... otherwise execute only if same context

    if { $evt != $powB3state || $cntxt==$powB3context } {
	set powB3state $evt
	set powB3context $cntxt
	eval $cmd
    }
}

proc powGetGraphOptions {gn} {
#returns list of extra graph options
    global powPlotParam

#test for requested graph    
    if {![info exists powPlotParam(xmagstep,$gn)] } {
	error "Graph $gn does not exist"
	return
    }

    lappend optlist "bgcolor" $powPlotParam(bgcolor,$gn)
    lappend optlist "xmargin" $powPlotParam(xmargin,$gn)
    lappend optlist "ymargin" $powPlotParam(ymargin,$gn)

    return $optlist
}


proc powGraphOptions {gn args} {
#add extra graph options here.  Arguments are "option value" pairs.
    global powPlotParam

#test for requested graph    
    if {![info exists powPlotParam(xmagstep,$gn)] } {
	error "Graph $gn does not exist"
	return
    }

    array set optionhash $args

    foreach option [array names optionhash] {
	switch $option {
	    bgcolor {
		set powPlotParam(bgcolor,$gn) $optionhash(bgcolor)
		.pow.pow itemconfigure ${gn}bkg -fill $optionhash(bgcolor) \
		    -outline $optionhash(bgcolor)
		.pow.pow lower ${gn}bkg
	    }
	    xmargin {
		set powPlotParam(xmargin,$gn) $optionhash(xmargin)
	    }
	    ymargin {
		set powPlotParam(ymargin,$gn) $optionhash(ymargin)
	    }
	    handletext {
		set powPlotParam(handletext,$gn) $optionhash(handletext)
		.pow.pow delete ${gn}handles
		.pow.pow delete ${gn}shandle
		powMakeGraphHandles $gn
	    }
	    handleanchor {
		set powPlotParam(handleanchor,$gn) $optionhash(handleanchor)
		.pow.pow delete ${gn}handles
		.pow.pow delete ${gn}shandle
		powMakeGraphHandles $gn
		powRestoreGraph $gn
	    }
	    handleposition {
		set powPlotParam(handleposition,$gn) \
		    $optionhash(handleposition)
		.pow.pow delete ${gn}handles
		.pow.pow delete ${gn}shandle
		powMakeGraphHandles $gn
		powRestoreGraph $gn
	    }
	    Grid {
		set powPlotParam(Grid,$gn) $optionhash(Grid)
		powAdornGraph $gn .pow.pow
	    }
	    GridColor {
		set powPlotParam(GridColor,$gn) $optionhash(GridColor)
		.pow.pow itemconfig ${gn}grid -fill $optionhash(GridColor)
	    }
	    GridDash {
		set powPlotParam(GridDash,$gn) $optionhash(GridDash)
		.pow.pow itemconfig ${gn}grid -dash $optionhash(GridDash)
	    }
	    NumTicks {
		set powPlotParam(NumTicks,$gn) $optionhash(NumTicks)
		powAdornGraph $gn .pow.pow
	    }
	    FixedAspect {
		set powPlotParam(FixedAspect,$gn) $optionhash(FixedAspect)
	    }
	    default {
		error "Unknown option $option"
	    }
	}
    }
}

proc powToplevel {topwin refwin {options ""}} {
#this implements what *should* be default behavior for X colormap handling.  
#Apparently the evil of Xlib colormap handling is contagious.
#if we're running in the plugin or in single window mode, this creates
#a frame instead of a toplevel.
    global powFrameForTop powbg powContainer

    
    if {!$powFrameForTop} {
	eval "toplevel $topwin -colormap $refwin -visual \"[winfo visual $refwin] [winfo depth $refwin]\" $options"
	focus $topwin
    } else {
#in a tclet just use frames
	frame $topwin -bg $powbg 

	if [info exists powContainer] {
	    grid $topwin -sticky news -in $powContainer
	} else {
	    grid $topwin -sticky news
	}
    }
}
    

proc powReditherImage {gn img {canvas .pow.pow} } {
#This recalculates the colors/intensities in the base Photo image.
#The call to powMagImage is necessary to make them appear.
    global powPseudoImages
#We could redither pseudocolors, but it isn't necessary
    if $powPseudoImages return 
    powReditherPhotoBlock $img
    powMagImage $gn $img $canvas
}


proc powReditherImages {gn img} {
#This checks whether we need to redither both a color bar and an image
#It also redithers the scope box if it is the current graph
    global powPseudoImages powPlotParam currgn powGUI

#We could redither pseudocolors, but it isn't necessary
    if $powPseudoImages return 

# Redither this image

    powReditherImage $gn $img

# Redither Scope

    if { $powGUI && $currgn == $gn } {
	powReditherImage ${gn}scope $img .pow.scope
    }

# Redither original image or colorbar if other exists

    if { [string match "*_colorbar" $gn] } {

	set origImage $powPlotParam(Colorbar,$gn)
	regexp "(.*)disp(.*)" $origImage z img gn
	powReditherImage $gn $img

    } else {
	if { [info exists powPlotParam(Colorbar,${gn}_colorbar)] } {
	    powReditherImage ${gn}_colorbar ${gn}_colorbar
	}
    }
}


proc powCmapInvert { } {
global powPseudoImages curr_img
    if $powPseudoImages {
	$curr_img colormap invert
    } else {
	powPhotoColorTable invert
    }
}

proc powSetRange { gn img min max} {
    global powPseudoImages powRBmin powRBmax
    set powRBmin(${img}disp${gn}) $min
    set powRBmax(${img}disp${gn}) $max

    if {$powPseudoImages} {
#bail out if some cretin (like myself:)  has zoomed in so far he actually 
#has no pixels left to display.  Also protects against "ribbon" images
#at magsteps < 1
	if {[image width ${img}disp${gn}] == 0  || \
		[image height ${img}disp${gn}] == 0 } return

	${img}disp$gn range $min $max

    } else {

	powReditherImage $gn $img

    }
}
    

proc powSetColorTable { } { 
    global cmap powLut powPseudoImages curr_img

    powSetLut $powLut
    if {$powPseudoImages}  {
	$curr_img colormap $cmap
    } else {
	powPhotoColorTable $cmap
    }
    invert_cmap_if_flag_set
}

proc powGenerateReplotCascade { } {
    .pow.edit.menu.replot delete 0 end
    foreach gn [powListGraphs] {
	if {![regexp {scope$} $gn]} {
	    .pow.edit.menu.replot add command -label "$gn" \
		-command "powMapGraph $gn"
	}
    }
}

proc powGenerateUnmapCascade { } {
    .pow.edit.menu.unmap delete 0 end
    foreach gn [powListGraphs] {
	if {![regexp {scope$} $gn]} {
	    .pow.edit.menu.unmap add command -label "$gn" \
		-command "powUnmapGraph $gn"
	}
    }
}

proc powGenerateMergeCascade { } {
    global currgn

    .pow.edit.menu.merge delete 0 end
    foreach gn [powListGraphs] {
	if {![regexp {scope$} $gn] && $currgn!=$gn} {
	    .pow.edit.menu.merge add command -label "$gn" \
		-command "powMergeGraphs $gn"
	}
    }
}

proc powProcessClick { x y binding} {
    global powClickCallback

    set gn [powWhereAmI [.pow.pow canvasx $x] [.pow.pow canvasy $y]] 
    if {$gn != "NULL"} {
	powSelectGraph $gn
	if [info exists powClickCallback] {
	    set gcoords [powCanvasToGraph $gn [.pow.pow canvasx $x] [.pow.pow canvasy $y]]
	    set rx [lindex $gcoords 0]
	    set ry [lindex $gcoords 1]
	    $powClickCallback $gn $rx $ry $binding
	}
    }
}
	
proc powPrint { type name orient } {
    set bbox [.pow.pow bbox all]
    set width  [expr [lindex $bbox 2]-[lindex $bbox 0]]
    set height [expr [lindex $bbox 3]-[lindex $bbox 1]]
    if [string match "Print*" $type] {
	set fnam "[pid]tmp.ps"
	.pow.pow postscript -colormode color -rotate $orient -file $fnam \
	    -width $width -height $height \
	    -x [lindex $bbox 0] -y [lindex $bbox 1]
	set comm "cat $fnam | $name"
	exec /bin/sh -c $comm
	catch {exec rm $fnam}
    } elseif [string match "File*" $type] {
	.pow.pow postscript -colormode color -rotate $orient -file $name \
	    -width $width -height $height \
	    -x [lindex $bbox 0] -y [lindex $bbox 1]
    }
    powShowHandles 1
}



proc powRescaleBox { } {
    global powbg curr_img currimg powRBmin powRBmax powbg
    if {![info exists curr_img]} {
	error "You must first select an image"
	return
    }

    powToplevel .powRB .pow "-bg $powbg -class \"POW Rescale\"" 
    
    

    label .powRB.min -text "Image min: $powRBmin($currimg)" -bg $powbg
    label .powRB.max -text "Image max: $powRBmax($currimg)" -bg $powbg

    
    label .powRB.cmin -text "Current min:" -bg $powbg
    entry .powRB.ecmin -textvariable powRBmin($curr_img) -relief sunken -bg $powbg

    label .powRB.cmax -text "Current max:" -bg $powbg
    entry .powRB.ecmax -textvariable powRBmax($curr_img) -relief sunken -bg $powbg
    
    frame .powRB.buttonFrame -bg $powbg
    button .powRB.apply -text Apply -command {
	powSetRange $currimggn $currimg \
		$powRBmin($curr_img) $powRBmax($curr_img)
	powSetRange ${currimggn}scope $currimg \
		$powRBmin($curr_img) $powRBmax($curr_img)
    }  -bg $powbg
    button .powRB.reset -text Reset -command {
	powSetRange $currimggn $currimg \
		$powRBmin($currimg) $powRBmax($currimg)
	powSetRange ${currimggn}scope $currimg \
		$powRBmin($currimg) $powRBmax($currimg)
    }  -bg $powbg
    button .powRB.exit -text Exit -command {destroy .powRB}  -bg $powbg

    grid configure .powRB.min -row 0 -column 0 -columnspan 2 -sticky w
    grid configure .powRB.max -row 0 -column 2 -columnspan 2 -sticky w
    grid configure .powRB.cmin -row 1 -column 0  -sticky w
    grid configure .powRB.ecmin -row 1 -column 1 -sticky w
    grid configure .powRB.cmax -row 1 -column 2  -sticky w
    grid configure .powRB.ecmax -row 1 -column 3  -sticky w
    grid configure .powRB.buttonFrame -row 2 -column 0 -columnspan 4 -sticky ew
    grid configure .powRB.apply -row 0 -column 0 -in .powRB.buttonFrame -sticky w
    grid configure .powRB.reset -row 0 -column 1 -in .powRB.buttonFrame
    grid configure .powRB.exit -row 0 -column 2 -in .powRB.buttonFrame -sticky e
}

proc powSetMagstepBox { } {
    global powPlotParam currgn powbg
    global powXMagstep powYMagstep powSaveXMagstep powSaveYMagstep

    if {![info exists currgn]} {
	error "You must first select a graph"
	return
    }
    if { [winfo exists .powMagstep] } {
	raise .powMagstep
	return
    }
    powToplevel .powMagstep .pow "-bg $powbg -class \"POW Magstep\"" 

    set powXMagstep $powPlotParam(xmagstep,$currgn) 
    set powYMagstep $powPlotParam(ymagstep,$currgn) 
    set powSaveXMagstep $powXMagstep
    set powSaveYMagstep $powYMagstep

    label .powMagstep.label -text "Current magstep:" -bg $powbg
    label .powMagstep.xlabel -text "X " -bg $powbg
    label .powMagstep.ylabel -text "Y " -bg $powbg
    entry .powMagstep.xmagstep -textvariable powXMagstep \
	    -relief sunken -bg $powbg
    entry .powMagstep.ymagstep -textvariable powYMagstep \
	    -relief sunken -bg $powbg
    
    frame .powMagstep.buttonFrame -bg $powbg
    button .powMagstep.apply -text Apply -command \
	{powMagGraph $currgn $powXMagstep $powYMagstep; \
	     set powXMagstep $powPlotParam(xmagstep,$currgn); \
	     set powYMagstep $powPlotParam(ymagstep,$currgn)}  -bg $powbg
    button .powMagstep.reset -text Reset -command \
	{set powXMagstep $powSaveXMagstep; \
	 set powYMagstep $powSaveYMagstep; \
	 powMagGraph $currgn $powXMagstep $powYMagstep}  -bg $powbg
    button .powMagstep.exit -text Exit -command {destroy .powMagstep}  -bg $powbg

    grid configure .powMagstep.label -row 0 -column 0 -sticky w
    grid configure .powMagstep.xlabel -row 0 -column 1
    grid configure .powMagstep.ylabel -row 1 -column 1
    grid configure .powMagstep.xmagstep -row 0 -column 2 -sticky w
    grid configure .powMagstep.ymagstep -row 1 -column 2 -sticky w
    grid configure .powMagstep.buttonFrame -row 2 -column 0 -columnspan 3 -sticky ew
    grid configure .powMagstep.apply -row 0 -column 0 -in .powMagstep.buttonFrame -sticky w
    grid configure .powMagstep.reset -row 0 -column 1 -in .powMagstep.buttonFrame
    grid configure .powMagstep.exit -row 0 -column 2 -in .powMagstep.buttonFrame -sticky e
}


proc powSetGraphSize { } {
    global powPlotParam currgn powbg
    global powXDim powYDim powSaveXDim powSaveYDim

    if {![info exists currgn]} {
	error "You must first select a graph"
	return
    }
    if { [winfo exists .powDim] } {
	raise .powDim
	return
    }
    powToplevel .powDim .pow "-bg $powbg -class \"POW Dim\"" 
    wm title .powDim "Set Graph Dimensions"

    set powXDim [tagXdim .pow.pow ${currgn}box]
    set powYDim [tagYdim .pow.pow ${currgn}box]
    set powSaveXDim $powXDim
    set powSaveYDim $powYDim

    label .powDim.label -text "Current Dim:" -bg $powbg
    label .powDim.xlabel -text "X " -bg $powbg
    label .powDim.ylabel -text "Y " -bg $powbg
    entry .powDim.xDim -textvariable powXDim \
	    -relief sunken -bg $powbg -width 10
    entry .powDim.yDim -textvariable powYDim \
	    -relief sunken -bg $powbg -width 10
    
    frame .powDim.buttonFrame -bg $powbg
    button .powDim.apply -text Apply -command \
	{powStretchGraphToSize $currgn $powXDim $powYDim}  -bg $powbg
    button .powDim.reset -text Reset -command \
	{set powXDim $powSaveXDim; \
	 set powYDim $powSaveYDim; \
	 powStretchGraphToSize $currgn $powXDim $powYDim}  -bg $powbg
    button .powDim.exit -text Exit -command {destroy .powDim}  -bg $powbg

    grid configure .powDim.label -row 0 -column 0 -sticky w
    grid configure .powDim.xlabel -row 0 -column 1
    grid configure .powDim.ylabel -row 1 -column 1
    grid configure .powDim.xDim -row 0 -column 2 -sticky w
    grid configure .powDim.yDim -row 1 -column 2 -sticky w
    grid configure .powDim.buttonFrame -row 2 -column 0 -columnspan 3 -sticky ew
    grid configure .powDim.apply -row 0 -column 0 -in .powDim.buttonFrame -sticky w
    grid configure .powDim.reset -row 0 -column 1 -in .powDim.buttonFrame
    grid configure .powDim.exit -row 0 -column 2 -in .powDim.buttonFrame -sticky e
}


proc powPrintBox { } {
    global powbg pcom fname pcom_fname powOutputType pcom_fname powPostOrient
    global powHandles
    powToplevel .powprint .pow "-bg $powbg -class \"POW Print\""
    

    set powOutputType "Printer Command:"
#orientation is 0-portrait 1-landscape
    set powPostOrient 0
    set pcom_fname "lpr"
    set pcom "lpr"
    set fname "pow.ps"
    set powHandles 0
    
    

    
    
	frame .powprint.frame -bg $powbg

	label .powprint.olabel \
		-anchor e \
		-text Output: \
                -width 16 \
                -bg $powbg

	radiobutton .powprint.pbutton \
		-text Printer \
                -value {Printer Command:} \
		-variable powOutputType \
                -command {set pcom_fname $pcom} \
                -bg $powbg

	radiobutton .powprint.fbutton \
		-text File \
		-value {File Name:} \
		-variable powOutputType \
                -command {set pcom_fname $fname} \
                -bg $powbg

	button .powprint.printbutton \
	-command {destroy .powprint; update idletasks; powPrint $powOutputType $pcom_fname $powPostOrient;} \
		-text Print/Save \
                -bg $powbg

	button .powprint.exitbutton \
                -command {destroy .powprint; powShowHandles 1} \
		-text Exit \
                -bg $powbg

	label .powprint.ocomlabel \
                -anchor e \
		-textvariable powOutputType \
	        -width 16   \
                -bg $powbg

	entry .powprint.pcomentry \
		-textvariable pcom_fname \
                -bg $powbg

	label .powprint.handlelabel \
		-anchor e \
		-text Orientation: \
	        -width 16   \
                -bg $powbg

        powShowHandles $powHandles

	radiobutton .powprint.showbutton \
		-text "Show Handles" \
		-value 1 \
		-variable powHandles \
                -command {powShowHandles 1} \
                -bg $powbg

	radiobutton .powprint.hidebutton \
		-text "Hide Handles" \
		-value 0 \
		-variable powHandles \
                -command {powShowHandles 0} \
                -bg $powbg

	label .powprint.orlabel \
		-anchor e \
		-text Orientation: \
	        -width 16   \
                -bg $powbg


	radiobutton .powprint.porbutton \
		-text Portrait \
		-value 0 \
		-variable powPostOrient \
                -bg $powbg

	radiobutton .powprint.landbutton \
		-text Landscape \
		-value 1 \
		-variable powPostOrient \
                -bg $powbg


	# Geometry management

	grid .powprint.frame -in .powprint	-row 5 -column 1  \
		-columnspan 3
	grid .powprint.olabel -in .powprint	-row 1 -column 1  \
		-sticky ew
	grid .powprint.pbutton -in .powprint	-row 1 -column 2 
	grid .powprint.fbutton -in .powprint	-row 1 -column 3 
	grid .powprint.printbutton -in .powprint.frame	-row 1 -column 1 
	grid .powprint.exitbutton -in .powprint.frame	-row 1 -column 3 
	grid .powprint.ocomlabel -in .powprint	-row 2 -column 1  \
		-sticky ew
	grid .powprint.pcomentry -in .powprint	-row 2 -column 2  \
		-columnspan 2
	grid .powprint.handlelabel -in .powprint -row 3 -column 1  \
		-sticky ew
	grid .powprint.showbutton -in .powprint	-row 3 -column 2 
	grid .powprint.hidebutton -in .powprint	-row 3 -column 3 
	grid .powprint.orlabel -in .powprint	-row 4 -column 1  \
		-sticky ew
	grid .powprint.porbutton -in .powprint	-row 4 -column 2 
	grid .powprint.landbutton -in .powprint	-row 4 -column 3 

	# Resize behavior management

	grid rowconfigure .powprint 1 -weight 0 -minsize 30
	grid rowconfigure .powprint 2 -weight 0 -minsize 30
	grid rowconfigure .powprint 3 -weight 0 -minsize 30
	grid rowconfigure .powprint 4 -weight 0 -minsize 50
	grid columnconfigure .powprint 1 -weight 0 -minsize 30
	grid columnconfigure .powprint 2 -weight 0 -minsize 30
	grid columnconfigure .powprint 3 -weight 0 -minsize 15

	grid rowconfigure .powprint.frame 1 -weight 0 -minsize 30
	grid columnconfigure .powprint.frame 1 -weight 0 -minsize 30
	grid columnconfigure .powprint.frame 2 -weight 1 -minsize 30
	grid columnconfigure .powprint.frame 3 -weight 0 -minsize 96


}


proc powMergeGraphs { newgn } {
    global currgn powPlotParam powGUI

    set curvelist ""
    if { $powPlotParam(curves,$currgn) != "NULL" } {
	eval lappend curvelist $powPlotParam(curves,$currgn)
    }
    if { $powPlotParam(curves,$newgn) != "NULL" } {
	eval lappend curvelist $powPlotParam(curves,$newgn)
    }
    if {$curvelist==""} {set curvelist "NULL"}

    set imagelist ""
    if { $powPlotParam(images,$currgn) != "NULL" } {
	eval lappend imagelist $powPlotParam(images,$currgn)
    }
    if { $powPlotParam(images,$newgn) != "NULL" } {
	eval lappend imagelist $powPlotParam(images,$newgn)
    }
    if {$imagelist==""} {set imagelist "NULL"}

    powCreateGraph $currgn $curvelist $imagelist \
      $powPlotParam(xunits,$currgn) $powPlotParam(yunits,$currgn)\
      $powPlotParam(xlabel,$currgn) $powPlotParam(ylabel,$currgn)\
      $powPlotParam(xdimdisp,$currgn) $powPlotParam(ydimdisp,$currgn)\
      NULL NULL NULL NULL
}




proc powAddTextToGraphDialog { } {
    global powbg
    powToplevel .powaddtext .pow "-bg $powbg -class \"POW Add Text\""
    label .powaddtext.graphlabel -text "Add to which graph?" -bg $powbg
    listbox .powaddtext.graphlist -relief sunken -bg $powbg
    foreach graph [powListGraphs] {
	.powaddtext.graphlist insert end $graph
    }
    label .powaddtext.entrylabel -text "Enter text:" -bg $powbg
    text .powaddtext.text -relief raised -bg $powbg -yscrollcommand ".powaddtext.scroll set" -width 20 -height 10
    scrollbar .powaddtext.scroll -command ".powaddtext.text yview" -bg $powbg
    label .powaddtext.instruct -text "Click OK and then use Left Mouse Button to place text." -bg $powbg
    button .powaddtext.ok -text "OK" -relief sunken -command {wm withdraw .powaddtext; powAddTextToGraphDoIt} -bg $powbg
    button .powaddtext.cancel -text "Cancel" -relief sunken -command "destroy .powaddtext" -bg $powbg 
    grid configure .powaddtext.graphlabel -row 0 -column 0 
    grid configure .powaddtext.graphlist -row 1 -column 0
    grid configure .powaddtext.entrylabel -row 0 -column 1
    grid configure .powaddtext.text -row 1 -column 1 -sticky nsew
    grid configure .powaddtext.scroll -row 1 -column 2 -sticky ns
    grid configure .powaddtext.instruct -row 2 -columnspan 3 -sticky w
    grid configure .powaddtext.ok -row 3 -column 0
    grid configure .powaddtext.cancel -row 3 -column 1
    grid rowconfigure .powaddtext 1 -weight 1
    grid columnconfigure .powaddtext 0 -weight 1
    

}


proc powAddTextToGraphDoIt { } {
    if [catch {selection get} gn] {
	set gn " "}
    set savebinding [bind .pow.pow <ButtonPress-1>]
    bind .pow.pow <ButtonPress-1> "\
       powPlaceText [list [.powaddtext.text get 0.0 end] $gn %x %y .pow.pow]; destroy .powaddtext;\
       bind .pow.pow <ButtonPress-1> \{$savebinding\}"
}

proc powPlaceText {text gn x y {canvas .pow.pow}} {
global powfont
     $canvas create text [$canvas canvasx $x] [$canvas canvasy $y] \
          -width 0 -font $powfont -text $text -anchor nw \
          -tags [list $gn ${gn}utext ${gn}text];
}




proc swap { a b} {
    upvar $a one
    upvar $b two
    set tmp $one
    set one  $two
    set two $tmp
}

 
proc debug_trace {name element op} { if {$element != ""} { set name
    ${name}($element) } upvar $name x puts "Variable $name set to $x"
}


proc powUpdateTrackVars {} { 
    global powGraphCoordsTracker powImagePixelTracker powImageValueTracker
    global powTrackText powFirstPixel currimg currgn powSixties


    if [regexp {[0-9]} $powTrackText(rx)] {
	if {$powSixties && [powWCSexists $currgn]} {
	    set rx  [powHourRA $powTrackText(rx)]
	    set ry  [powDegDec $powTrackText(ry)]
	    set powGraphCoordsTracker \
		"Graph coordinates: ( $rx, $ry )"
	} else {
	    set powGraphCoordsTracker \
		[format "Graph coordinates: ( %.6lg, %.6lg )" \
		     $powTrackText(rx) $powTrackText(ry)]
	}
    } else {
	set powGraphCoordsTracker "Graph coordinates: ( X , X )"
    }


    if [regexp {[0-9]} $powTrackText(imgx)] {
	set powImagePixelTracker "Image pixel: ( [expr $powTrackText(imgx) + $powFirstPixel], [expr $powTrackText(imgy) + $powFirstPixel] )"
    } else {
	set powImagePixelTracker "Image pixel: ( X , X )"
    }
	
    if [regexp {[0-9]} $powTrackText(imgz)] {
	set pixval [format %.16lg [expr $powTrackText(imgz)]]
    } else {
	set pixval "X"
    }
    set powImageValueTracker "Pixel value: $pixval ($powTrackText(zunits))"

}





proc powHelp {topic} {
    global env powHelpTopics powbg  Url
    if { [winfo exist .pow.hyperhelp] == 0} {
	powToplevel .pow.hyperhelp .pow "-class \"POW Help\"  -bg $powbg"
	button .pow.hyperhelp.quit -command {destroy .pow.hyperhelp} -bg $powbg -text Quit
	label .pow.hyperhelp.title -text $topic -bg $powbg
	text .pow.hyperhelp.text -yscrollcommand ".pow.hyperhelp.scrollbar set" -bg $powbg
	scrollbar .pow.hyperhelp.scrollbar -command ".pow.hyperhelp.text yview" -bg $powbg
	grid .pow.hyperhelp.quit -row 0 -column 0 -sticky w
	grid .pow.hyperhelp.title -row 1 -column 0 -columnspan 2
	grid .pow.hyperhelp.text -row 2 -column 0 -sticky news
	grid .pow.hyperhelp.scrollbar -row 2 -column 1 -sticky nsw
	grid rowconfigure .pow.hyperhelp 2 -weight 1
	grid columnconfigure .pow.hyperhelp 0 -weight 1
	HMinit_win .pow.hyperhelp.text
    }
    set Url "$env(TCLTK_LIBRARY)/pow/$topic"
    render $Url
}

# This stuff was "stolen" from the sample program in the Hippo distribution
# go render a page.  We have to make sure we don't render one page while
# still rendering the previous one.

proc render {file} {
	global HM.pow.hyperhelp.text
	HMreset_win .pow.hyperhelp.text
	update idletasks
        set html [get_html $file]
        HMparse_html $html {HMrender .pow.hyperhelp.text}
	HMset_state .pow.hyperhelp.text -stop 1		;
# stop rendering previous page if busy
}

# given a file name, return its html

proc get_html {file} {
	global Home
	if {[catch {set fd [open $file]} msg]} {
		return "
			<title>Bad file $file</title>
			<h1>Error reading $file</h1><p>
			$msg<hr>
		"
	}
	set result [read $fd]
	close $fd
	return $result
}


# Override the library link-callback routine for the sample app.
# It only handles the simple cases

proc HMlink_callback {win href} {
	global Url
	if {[string match /* $href]} {
		set Url $href
	} else {
		set Url [file dirname $Url]/$href
	}
	update
	render $Url
}

# supply an image callback function
# Read in an image if we don't already have one
# callback to library for display

proc HMset_image {handle src} {
	global Url message
	if {[string match /* $src]} {
		set image $src
	} else {
		set image [file dirname $Url]/$src
	}
	set message "fetching image $image"
	update
	if {[string first " $image " " [image names] "] >= 0} {
		HMgot_image $handle $image
	} else {
		catch {image create photo $image -file $image} image
		HMgot_image $handle $image
	}
}


#end of code "stolen" from Hippo sample


proc powMax { a b } {
    return [expr ($a > $b) ? $a : $b]
}

proc powMin { a b } {
    return [expr ($a < $b) ? $a : $b]
}

proc powFindFont { w {pointsizes 120} {weight medium} {slant r}} {
    foreach family {times courier helvetica } {
	foreach points $pointsizes {
	    if {[catch {$w config -font \
			    -*-$family-$weight-$slant-*-*-*-$points-*}] == 0} {
		return -*-$family-$weight-$slant-*-*-*-$points-*
	    }
	}
    }
    $w config -font fixed
    return fixed
}

proc powScopeZoom {in_or_out } {
    global currgn saveROI
    if {! [string compare [.pow.scope find  withtag ROI] ""]} {
	# no ROI currently, ROI is whole graph
	set ROIbbox [.pow.scope coords ${currgn}scopebox]
    } else {
	set ROIbbox [.pow.scope coords ROI]
	set saveROI $ROIbbox
	.pow.scope delete ROI
    }
    if [regexp "in" $in_or_out] {
	set multiplier 0.5
    } else {
	set multiplier 2.0
    }
    set halfwidth [expr ([lindex $ROIbbox 2] - [lindex $ROIbbox 0])/2.0]
    set halfheight [expr ([lindex $ROIbbox 3] - [lindex $ROIbbox 1])/2.0]
    set x_center [expr [lindex $ROIbbox 0] + $halfwidth]
    set y_center [expr [lindex $ROIbbox 1] + $halfheight]
    set new_halfwidth [expr $halfwidth * $multiplier]
    set new_halfheight [expr $halfheight * $multiplier]
    .pow.scope create rectangle [expr $x_center - $new_halfwidth] [expr $y_center - $new_halfheight] [expr $x_center + $new_halfwidth] [expr $y_center + $new_halfheight] -tags ROI -outline blue
    powEndROI 0 .pow.scope
}



proc powSaveConfig { } {
    global powfont powbg cmap cmap_inv powLut powCurveParam
    global powcursor powResizeMain
    global showlinks powScopeHeight powScopeWidth powMinHeight powSixties
    global powNumTicks powGrid powGridColor powGridDash
    if [catch {open "~/.powrc" w} RCFILE] {
	error "Couldn't open ~/.powrc, not saving configuration"
    } else {
	puts $RCFILE "set powfont \"$powfont\""
	puts $RCFILE "set powbg \"$powbg\""
	puts $RCFILE "set cmap \"$cmap\""
	puts $RCFILE "set cmap_inv \"$cmap_inv\""
	puts $RCFILE "set powLut \"$powLut\""
	puts $RCFILE "set powcursor \"$powcursor\""
	puts $RCFILE "set powResizeMain \"$powResizeMain\""
	puts $RCFILE "set showlinks \"$showlinks\""
	puts $RCFILE "set powScopeWidth \"$powScopeWidth\""
	puts $RCFILE "set powScopeHeight \"$powScopeHeight\""
	puts $RCFILE "set powMinHeight \"$powMinHeight\""
	puts $RCFILE "set powSixties \"$powSixties\""
	puts $RCFILE "set powNumTicks \"$powNumTicks\""
	puts $RCFILE "set powGrid \"$powGrid\""
	puts $RCFILE "set powGridColor \"$powGridColor\""
	puts $RCFILE "set powGridDash \"$powGridDash\""

	foreach opt $powCurveParam(allOpts,powDef) {
	    puts $RCFILE "set powCurveParam($opt,powDef) \"$powCurveParam($opt,powDef)\""
	}

	close $RCFILE
    }
}

proc powBreakAllLinks { } {
#deletes all link info
    global axisToChainHash chainToAxisHash nextchain
 
    if [info exists axisToChainHash] {
	unset axisToChainHash 
	unset chainToAxisHash 
	unset nextchain
    }
    .pow.pow delete link
}
    


proc powLinkAxes {gn1 axis1 gn2 axis2} {
    global axisToChainHash chainToAxisHash nextchain

#catch stupid input
    if {$gn1 == $gn2} {return}
    

    if {![info exists nextchain]} {
	set nextchain 1
    }
    set graphlist [powListGraphs] 

    if {[lsearch $graphlist $gn1] == -1} {return "graph $gn1 does not exist";}
    if {[lsearch $graphlist $gn2] == -1} {return "graph $gn2 does not exist";}

    set chain1 0
    set chain2 0
    
    if {[array names axisToChainHash $gn1$axis1] != ""} {set chain1 $axisToChainHash($gn1$axis1)}
    if {[array names axisToChainHash $gn2$axis2] != ""} {set chain2 $axisToChainHash($gn2$axis2)}
    if {$chain1 == 0 && $chain2 == 0} {
#new chain
	set axisToChainHash($gn1$axis1) $nextchain
	set axisToChainHash($gn2$axis2) $nextchain
	set chainToAxisHash($nextchain) [list $gn1$axis1 $gn2$axis2]
	incr nextchain
    } elseif {$chain1 != 0 && $chain2 !=0} {
#two chains, delete one
	powMergeChains $chain1 $chain2
    } elseif {$chain1 != 0} {
#add axis 2 to chain 1	
	set axisToChainHash($gn2$axis2) $chain1
	lappend chainToAxisHash($chain1) $gn2$axis2
    } else {
#add axis 1 to chain 2
	set axisToChainHash($gn1$axis1) $chain2
	lappend chainToAxisHash($chain2) $gn1$axis1
    }
}

proc powMergeChains {chain1 chain2} {
#delete two existing chains and make a new one consisting of all of their
#members    
    global axisToChainHash chainToAxisHash nextchain
    foreach axis [array names axisToChainHash] {
	if {$axisToChainHash($axis) == $chain1 || \
		$axisToChainHash($axis) == $chain2} {
	    set axisToChainHash($axis) $nextchain
	}
    }
    set chainToAxisHash($nextchain) [concat $chainToAxisHash($chain1) $chainToAxisHash($chain2)]
    unset chainToAxisHash($chain1)
    unset chainToAxisHash($chain2)
}
    
 
proc powBreakLink {gn axis} {
#removes graph 1 axis 1 from the chain it belongs to
    global axisToChainHash chainToAxisHash
    set chain 0
    if {[array names axisToChainHash $gn$axis] != ""} {
	set chain $axisToChainHash($gn$axis)
    } else {
	puts "Graph $gn axis $axis is not part of a chain"
	return
    }
    unset axisToChainHash($gn$axis)
    if {[llength $chainToAxisHash($chain)] <= 2} {
	unset chainToAxisHash($chain)
    } else {
	set chainToAxisHash($chain) [lreplace $chainToAxisHash($chain) [set bozo [lsearch $chainToAxisHash($chain) $gn$axis]] $bozo]
    }
}

proc chop {theString} {
    return [string range $theString 0 [expr [string length $theString]-2]]
}
    

proc powAlignChain {gn axis orient} {
#stacks all graphs in a chain either (H)orizontally or (V)ertically
    global axisToChainHash chainToAxisHash powResizeMain powPlotParam   
    if {![info exists axisToChainHash($gn$axis)]} return;
    set oldgraph [lindex $chainToAxisHash($axisToChainHash($gn$axis)) 0]
    set oldgraph [chop $oldgraph]
    foreach graph [lrange $chainToAxisHash($axisToChainHash($gn$axis)) 1 end] {
	set graph [chop $graph]
	set gbox [.pow.pow coords ${oldgraph}box]
	set bbox [.pow.pow bbox ${oldgraph}]
	if {[lindex $orient 0] == "H"} {
	    set toX [expr [lindex $bbox 2] + 2 * $powPlotParam(xmargin,$graph) ]
	    set toY [lindex $gbox 1]
	} else {
	    set toX [lindex $gbox 0]
	    set toY [expr [lindex $bbox 3] + 2 * $powPlotParam(ymargin,$graph) ]
	}
	set coords [.pow.pow coords ${graph}box]
	set fromX [lindex $coords 0] 
	set fromY [lindex $coords 1] 
	powMoveGraph $graph [expr int($toX - $fromX)] [expr int($toY - $fromY)]
	set oldgraph $graph
    }
    powReconfigureToplevel $powResizeMain
}


proc powReconfigureToplevel {{resizemain 1}  } {
    global powHeaderHeight powMinHeight powMinWidth powMaxWidth powMaxHeight
    global powContainer
#resize POW window if necessary
#    update idletasks
    set bigbbox [.pow.pow bbox all]
    if {$resizemain &&  ($powContainer == "none" || $powContainer == "NULL")} {
	set windowX [expr [lindex $bigbbox 2] + 50]
	set windowY [expr [lindex $bigbbox 3] + $powHeaderHeight + 50]
	set windowY [expr {$windowY < $powMinHeight} ? $powMinHeight : $windowY]
	set windowX [expr {$windowX < $powMinWidth} ? $powMinWidth : $windowX]
	set windowY [expr {$windowY > [expr $powMaxHeight + $powHeaderHeight]}\
			 ? [expr $powMaxHeight + $powHeaderHeight] : $windowY]
	set windowX [expr {$windowX > $powMaxWidth} ? $powMaxWidth : $windowX]
	set x [winfo x .pow]
	set y [winfo y .pow]
	if {$x != 0 && $y != 0} {
	    catch {wm geometry .pow "${windowX}x${windowY}+$x+$y"}
	} else {
	    catch {wm geometry .pow "${windowX}x${windowY}"}
	}
    }

    # Check if scrollregion has significantly changed so that one doesn't
    # force a full-screen update

    set currBnds [.pow.pow cget -scrollregion]
    set newBnds [list [expr [lindex $bigbbox 0] - 20] \
	              [expr [lindex $bigbbox 1] - 20] \
	              [expr [lindex $bigbbox 2] + 20] \
	              [expr [lindex $bigbbox 3] + 20] ]

    if { [expr abs([lindex $currBnds 0]-[lindex $newBnds 0])]>10 \
	    || [expr abs([lindex $currBnds 1]-[lindex $newBnds 1])]>10 \
	    || [expr abs([lindex $currBnds 2]-[lindex $newBnds 2])]>10 \
	    || [expr abs([lindex $currBnds 3]-[lindex $newBnds 3])]>10 } {
	.pow.pow configure -scrollregion $newBnds
    }

    powShowLinks
}
    


proc powChangeBg {} {
    global powbg powShowHandlesFlag

    foreach com [info commands .pow.*] {
	catch {$com configure  -bg $powbg}
	catch {$com configure -background $powbg}
    }

    if {$powShowHandlesFlag} then {
	.pow.pow itemconfigure buttonbkg -fill $powbg 
	.pow.pow lower buttonbkg
    }

#catch the next line in case no graphs yet    
    catch {.pow.currgn configure -background yellow}

    .pow configure -bg $powbg
#    tk_setPalette $powbg

}

proc powChangeFontBox {} {
    global powfont powbg
    
    set savefont $powfont
    
    powToplevel .powfont  .pow "-bg $powbg"
    
    label .powfont.label -text "Enter a new X font name: " -bg $powbg
#   in case somebody tried for a font that doesn't exist, catch error
    if [catch {.pow.testfont config -font $powfont} ] {
	set powfont [powFindFont .pow.testfont [list 120 130] ]
    }
    
    entry .powfont.entry -relief sunken -bd 2 -textvariable powfont -bg $powbg -font $powfont -width 50 
    button .powfont.default -text "Default" -command {set powfont [powFindFont .pow.testfont [list 120 130] ]}  -bg $powbg
    button .powfont.ok -text "OK" -command {destroy .powfont; powChangeFont} -bg $powbg
    button .powfont.apply -text "Apply" -command {powChangeFont} -bg $powbg
    button .powfont.cancel -text "Cancel" -command "set powfont $savefont; powChangeFont; destroy .powfont" -bg $powbg
    
    pack .powfont.label .powfont.entry .powfont.default .powfont.apply .powfont.ok .powfont.cancel -side left -padx 1m -pady 2m
    
    bind .powfont.entry <Return> {destroy .powfont; powChangeFont}
}

proc powChangeFont {} {
    global powfont

    catch {.powfont.entry config -font $powfont}
    foreach gn [powListGraphs] {
	.pow.pow itemconfigure ${gn}text -font $powfont
    }
    .pow.pow itemconfigure text -font $powfont
}




proc powExit { }  {
global axisToChainHash chainToAxisHash nextchain powGUI
    catch { unset axisToChainHash chainToAxisHash nextchain}
    event generate .pow <<powExit>>
    destroy .pow
    powCleanUp
}

proc powUnmapGraph { gn {scope 1}} {
    global powGUI currgn

#Removes a graph from the display
    .pow.pow delete $gn
    .pow.pow delete ${gn}handles
    if { [info exists currgn] && $currgn==$gn } {
	set currgn ""
	.pow.pow delete current_gn
	if {$powGUI && $scope} {
	    .pow.scope delete all
	}
    }
}

proc powMapGraph { gn {restore_position 0}} {
#puts a graph back on the display
    global powPlotParam

    if $restore_position {
	set xo $powPlotParam(xo,$gn) 
	set yo $powPlotParam(yo,$gn)
    }


    powCreateGraph $gn $powPlotParam(curves,$gn) $powPlotParam(images,$gn) \
	$powPlotParam(xunits,$gn) $powPlotParam(yunits,$gn) \
	$powPlotParam(xlabel,$gn) $powPlotParam(ylabel,$gn) \
	$powPlotParam(xdimdisp,$gn) $powPlotParam(ydimdisp,$gn) \
	$powPlotParam(xBot,$gn) $powPlotParam(yBot,$gn) \
	$powPlotParam(xTop,$gn) $powPlotParam(yTop,$gn) 
	

    if $restore_position {
	powMoveGraphTo $gn $xo $yo .pow.pow
    }
}

proc powDeleteImage  {gn img} {
#deletes 
    global powPlotParam 
    .pow.pow delete ${img}disp$gn
    catch {.pow.scope delete ${img}disp${gn}scope}
    .pow.pow delete current_img
    set whichImage [lsearch $powPlotParam(images,$gn) $img] 
    set powPlotParam(images,$gn) [lreplace $powPlotParam(images,$gn) $whichImage $whichImage]
}

proc powDeleteCurve  {gn curve} {
#deletes 
    global powPlotParam 
    .pow.pow delete ${curve}$gn
    catch {.pow.scope delete ${curve}${gn}scope}
    set whichCurve [lsearch $powPlotParam(curves,$gn) $curve] 
    set powPlotParam(curves,$gn) [lreplace $powPlotParam(curves,$gn) $whichCurve $whichCurve]
}


proc invert_cmap_if_flag_set {} {
    global cmap_inv curr_img powPseudoImages
    if {$cmap_inv} then {
	if $powPseudoImages {
	    $curr_img colormap invert
	} else {
	    powPhotoColorTable invert
	}
    }
}

proc powCmapStretchIntensity { cwidth clen lut} {
    global powPseudoImages curr_img powCurrentLut currimg currimggn currgn
    global powPlotParam
    
    if $powPseudoImages {
	eval {$curr_img cmap_stretch intensity $cwidth $clen} $lut
    } else {
	eval {powPhotoCmapStretch $cwidth $clen} $lut
    }
}
	
	
proc powSetLut {lut} {
    global curr_img powCurrentLut powClen powSqueeze powSlide powLut
    global currimg currgn powPlotParam
    set powClen 245
    
    if {![info exists curr_img]} return
    
    if { [string match "*_colorbardisp*_colorbar" $curr_img] } {
	regexp "(.*)disp(.*)" $curr_img z img gn
	set ImageLut $powPlotParam(Colorbar,$gn)
    } else {
	set ImageLut $curr_img
    }
    if {$lut == "current"} {
	if {[info exists powCurrentLut($ImageLut)]} {
	    powCmapStretchIntensity $powClen $powClen $powCurrentLut($ImageLut)
	    return
	} else {
	    set lut $powLut
	}
    }
#Note, current version of visu only allows 50 points in a user-defined lut
#NOTE!!! the y coordinate for lut's is X Windows-like, i.e. the origin
#is in the upper left
    
    set powCurrentLut($ImageLut) {}
    if {$lut == "linear"} then {
 #This could be done with 2 pts, but interactive colormap diddling needs
 # a full array to work on
  	for {set i 0} {$i <= $powClen} {incr i 5} {
  	    lappend powCurrentLut($ImageLut) $i
  	    lappend powCurrentLut($ImageLut) [expr $powClen - $i]
  	}
	
    } elseif {$lut == "sqrt"} then {
  	for {set i 0} {$i <= $powClen} {incr i 5 } {
  	    lappend powCurrentLut($ImageLut) $i
  	    lappend powCurrentLut($ImageLut) [expr $powClen - int(round((sqrt($i))/sqrt($powClen) * $powClen))]
  	}
    } elseif {$lut == "log"} then {
  	set denom [expr log10($powClen+1.0)]
  	for {set i 0} {$i <= $powClen} {incr i 5 } {
  	    lappend powCurrentLut($ImageLut) $i
 	    lappend powCurrentLut($ImageLut) [expr int(round( $powClen * (1.0 - (log10($i+1.0)/$denom))))]
	}
    }
#    puts "\n$powCurrentLut($ImageLut)"
    powCmapStretchIntensity $powClen $powClen $powCurrentLut($ImageLut)

    set powSqueeze 0.0
    set powSlide 0.0
}

proc powBoundDiddleLut {gn x y} {
    set cx [.pow.pow canvasx $x]
    set cy [.pow.pow canvasy $y]
    set bbox [.pow.pow coords ${gn}box]
    set lx [lindex $bbox 0]
    set ly [lindex $bbox 1]
    set ux [lindex $bbox 2]
    set uy [lindex $bbox 3]
#make range from -1 to 1
    set fx [expr (2.0 * ($cx - $lx)/($ux - $lx) - 1.0)]
    set fy [expr (2.0 * ($cy - $ly)/($uy - $ly) - 1.0)]

    powDiddleLut $fx $fy

}
    


proc powDiddleLut { slide squeeze } {
#$squeze and $slide should range from -1 to 1, not inclusive
    global curr_img powCurrentLut powClen powSqueeze powSlide currimg currimggn
    global powPseudoImages powPlotParam currgn

    set powSqueeze $squeeze
    set powSlide $slide

    if {! [info exists powClen]} return

    if { [string match "*_colorbardisp*_colorbar" $curr_img] } {
	regexp "(.*)disp(.*)" $curr_img z img gn
	set ImageLut $powPlotParam(Colorbar,$gn)
    } else {
	set ImageLut $curr_img
    }

    set squeeze [expr double($powSqueeze)]
    set slide [expr double($powSlide)]
    set increment [expr int($powClen * $slide)]

    if {$squeeze > 0 && $squeeze < 1} {
	set factor [expr (1.0 - $squeeze)] 
	set increment [expr $increment + $squeeze*$powClen/5.0]
    } elseif {$squeeze < 0 && $squeeze > - 1} {
	set factor [expr 1.0/(1.0 + $squeeze)] 
	set increment [expr $increment - ($factor - 1.0) * $powClen/5.0]
    } else {
	set factor 1.0
    }

    for {set i 0} {$i <= $powClen} {incr i 5} {
	set newi [expr $i * $factor + $increment]
	set newi [expr ($newi < 0 ) ? 0 : $newi]
	set newi [expr ($newi > $powClen ) ? $powClen : $newi]
	lappend l2 [expr int(floor($newi))]
	lappend l2 [lindex $powCurrentLut($ImageLut) [expr 2*$i/5+1]]
    }
    set powCurrentLut($ImageLut) $l2

    powCmapStretchIntensity $powClen $powClen $l2
    powReditherImages $currimggn $currimg
    
}

proc powShowHandles {showhandles } {
    global powShowHandlesFlag currgn powbg powPlotParam
    set powShowHandlesFlag $showhandles
    if {$showhandles} then {
	.pow.pow raise current_img
	.pow.pow itemconfigure current_img -outline green
	.pow.pow itemconfigure current_gn -outline yellow
	.pow.pow raise buttonfg
	.pow.pow configure -bg $powbg
	foreach graph [powListGraphs] {
	    .pow.pow itemconfigure ${graph}bkg -fill $powPlotParam(bgcolor,$graph) \
		-outline $powPlotParam(bgcolor,$graph)
	}
	foreach wind [.pow.pow find withtag canvas_window] {
	    set windowname [.pow.pow itemcget $wind -window]
	    $windowname configure -foreground black -background \
		$powPlotParam(bgcolor,$graph)  \
		-highlightthickness 1 -relief raised
	}
    } else {
	.pow.pow itemconfigure ohandle -outline white
	.pow.pow itemconfigure graphbkg -fill white -outline white
	.pow.pow configure -bg white
	.pow.pow raise ${currgn}line
	foreach wind [.pow.pow find withtag canvas_window] {
	    set windowname [.pow.pow itemcget $wind -window]
	    $windowname configure -foreground white -background white \
		-highlightthickness 0 -relief flat
	}
    }
}

proc chopped {theString} {
    return [string index $theString [expr [string length $theString] - 1]]
}


proc powShowLinks { } {
    global showlinks axisToChainHash chainToAxisHash
    .pow.pow delete link
    if {$showlinks} then {
	foreach chain [array names chainToAxisHash] {
	    set oldaxis [chopped $chainToAxisHash($chain)]
	    set oldgraph [chop [lindex $chainToAxisHash($chain) 0]]
	    foreach graph [lrange $chainToAxisHash($chain) 1 end] {
		set axis [chopped $graph]
		set graph [chop $graph]
		set abox [.pow.pow coords ${oldgraph}box]
		set bbox [.pow.pow coords ${graph}box]
		if {$oldaxis == "X"} {
		    set fromX [expr int(([lindex $abox 2]+[lindex $abox 0])/2.0)]
		    set fromY [expr int([lindex $abox 3])]
		} else {
		    set fromY [expr int(([lindex $abox 3]+[lindex $abox 1])/2.0)]
		    set fromX [expr int([lindex $abox 0])]
		}
		if {$axis == "X"} {
		    set toX [expr int(([lindex $bbox 2]+[lindex $bbox 0])/2.0)]
		    set toY [expr int([lindex $bbox 3])]
		} else {
		    set toY [expr int(([lindex $bbox 3]+[lindex $bbox 1])/2.0)]
		    set toX [expr int([lindex $bbox 0])]
		}
		.pow.pow create line $fromX $fromY $toX $toY -tags "link" -fill pink
		set oldaxis $axis
		set oldgraph $graph
	    }
	}
    }
}


proc powPlotImages {gn images {canvas ".pow.pow"}} {
    global powPlotParam cmap
    global filename_array powcursor
    global powPseudoImages curr_img powRBmin powRBmax


    if [regexp "NULL" $images] return

    .pow configure -cursor watch
    .pow.pow configure -cursor watch
    catch {.pow.scope configure -cursor watch}

#   remove "NULL" from images list if present
    if [regexp "NULL" $powPlotParam(images,$gn)] {set powPlotParam(images,$gn) { }}

    set imgcnt 0
    foreach current_image "$images" {

#   if image is already in list, don't plot it
	if {[lsearch $powPlotParam(images,$gn) $current_image]>=0} continue

	incr imgcnt

#	puts "\nplotting image: $current_image in $gn"


#make a copy of the current image 
	if $powPseudoImages {
	    image create pict ${current_image}disp$gn
	} else {
	    image create photo ${current_image}disp$gn
	}

#This if block allows rescalings to persist through ROI zooms
#you could get very bizarre behavior is somebodies reusing image names....
#nothing I can think of to do about that though
	if {![info exists powRBmin(${current_image}disp$gn)]} {
	    set powRBmin(${current_image}disp$gn) $powRBmin($current_image)
	    set powRBmax(${current_image}disp$gn) $powRBmax($current_image)
	}

	if {!$powPseudoImages} {
#Setup the ColorTable for Photo images
	    set curr_img ${current_image}disp$gn
	    if { ![string match "*_colorbar" $gn] } {
		powSetColorTable
	    }
	    powSetLut current
	}


	set clipbox [powGetImageClipbox $gn $current_image $canvas]
	set powPlotParam(clipbox$current_image,$gn) $clipbox

        if {![regexp "clipped" $clipbox]}  {

	    # First two elements indicate location on graph to place image
	    set x0 [lindex $clipbox 0]
	    set y0 [lindex $clipbox 1]
	    set pcoords [powGraphToCanvas $gn $x0 $y0 $canvas]
	    set x0 [lindex $pcoords 0]
	    set y0 [lindex $pcoords 1]


	    set image_id [$canvas create image $x0 $y0 \
			      -image ${current_image}disp$gn -anchor sw \
			      -tags "$gn disp$gn frame${imgcnt}$gn ${current_image}disp$gn image_body img_$current_image"]


	    if {$canvas == ".pow.pow" } {
		.pow.pow bind $image_id <Button-1> [list powSelectImage $gn $current_image]
		.pow.pow bind $image_id <B1-Motion> "powBoundDiddleLut $gn \%x \%y"
		if { ![info exists selImg] } {set selImg $current_image}
	    }
	}
	lappend powPlotParam(images,$gn) $current_image 
	powMagImage $gn $current_image $canvas
    }
    if { [info exists selImg] } {powSelectImage $gn $selImg}

#   powMagGraph $gn $powPlotParam(xmagstep,$gn) \
#                   $powPlotParam(ymagstep,$gn) $canvas
#   powRedrawBox $gn $canvas
#   powAdornGraph $gn $canvas

    .pow configure -cursor $powcursor
    .pow.pow configure -cursor $powcursor
    catch {.pow.scope configure -cursor $powcursor}
}


proc powDeSelectImage { } {
    global curr_img currimg currimggn powPlotParam
    if {[string compare [.pow.pow find  withtag current_img] ""]} {
	.pow.pow delete current_img
    }
    catch {unset curr_img}
    catch {unset currimg}
    catch {unset currimggn}
}

proc powSelectImage {gn img} {
    global curr_img currimg currimggn powPlotParam powGUI currgn

#delete previous bbox rectangle
    .pow.pow delete current_img 
#make all things visible
    .pow.pow raise ${gn}line

    set curr_img ${img}disp$gn
    set currimg $img
    set currimggn $gn
    set powPlotParam(currimg,$gn) $img

    set tags [.pow.pow find withtag disp$gn]
    if { $tags != "" } {
	.pow.pow raise $curr_img [lindex $tags end]
    }
    if $powGUI {
	set scopeids [.pow.scope find withtag disp${gn}scope]
	if {$scopeids != ""} {
	    .pow.scope raise img_$img [lindex $scopeids end]
	}
    }

    set ibbox [.pow.pow bbox ${img}disp$gn]
    if {$ibbox != ""} {
	eval [concat .pow.pow create rectangle  $ibbox -tags [list "current_img $gn handle ohandle"] -outline green]
    }

    if { [info exists currgn] &&  $gn == $currgn } {
	powSetLut current
    }

}

proc powSelectGraph {gn} {
    global currgn mag powPlotParam powScopeWidth powScopeHeight
    global powGrid powGridColor powGridDash powGUI

#add a bit of slack at the lower right corner
    set bbox [.pow.pow bbox $gn]
    set bbox [lreplace $bbox 2 2 [expr [lindex $bbox 2] + 2]]
    set bbox [lreplace $bbox 3 3 [expr [lindex $bbox 3] + 2]]
    set oldbbox [.pow.pow coord current_gn]
    
    if { $oldbbox=="" || \
      [expr abs( [lindex $bbox 0] - [lindex $oldbbox 0] ) + \
	    abs( [lindex $bbox 1] - [lindex $oldbbox 1] ) + \
	    abs( [lindex $bbox 2] - [lindex $oldbbox 2] ) + \
	    abs( [lindex $bbox 3] - [lindex $oldbbox 3] ) ] > 1 } {
#delete previous bbox rectangle
	.pow.pow delete current_gn
	eval [concat .pow.pow create rectangle  $bbox \
		-tags [list "current_gn ${gn}yhandle handle ohandle"] \
		-outline yellow -width 3]
    }

# If we are reselecting the current graph, just return
    if { [info exists currgn] && $currgn==$gn } return
    set currgn $gn

    if {![regexp "NULL" $powPlotParam(images,$gn)]} {
# Save active image for later
	if {[info exists powPlotParam(currimg,$gn)]} \
		{ set saveCurrImg $powPlotParam(currimg,$gn) }
    }

    if $powGUI {
	if {! [string compare [.pow.scope find  withtag ${gn}scopebox] ""]} {
	    .pow.scope delete all
	    if {[regexp "NULL" $powPlotParam(curves,$gn)] && \
		    [regexp "NULL" $powPlotParam(images,$gn)] } return
	    powCreateGraph ${gn}scope $powPlotParam(curves,$gn) \
		$powPlotParam(images,$gn) $powPlotParam(xunits,$gn) \
		$powPlotParam(yunits,$gn) $powPlotParam(xlabel,$gn) \
		$powPlotParam(ylabel,$gn) \
		[expr $powScopeWidth - 40] \
		[expr $powScopeHeight - 40] \
		NULL NULL NULL NULL .pow.scope
	    powDrawScopeROI [list \
		    $powPlotParam(xBot,$gn) \
		    $powPlotParam(yBot,$gn) \
		    $powPlotParam(xTop,$gn) \
		    $powPlotParam(yTop,$gn)]
	}
    }
    
# Now restore current image for this graph
    if {[info exists saveCurrImg]} { powSelectImage $gn $saveCurrImg }
    
    .pow.pow raise ${gn}handles
    .pow.pow lower ${gn}bkg
    .pow.pow raise DrawRegion
    .pow.pow raise rgnHandle
    .pow.pow raise $gn

    set powGrid $powPlotParam(Grid,$gn)
    set powGridColor $powPlotParam(GridColor,$gn)
    set powGridDash $powPlotParam(GridDash,$gn)
    set powNumTicks $powPlotParam(NumTicks,$gn)

}


proc powMagImage {gn img {canvas .pow.pow}} {
#lowlevel routine, don't call this yourself
#resizes image to match the current magstep and ROI window
    global powPlotParam cmap cmap_inv powLut curr_img currimg currimggn
    global powPseudoImages powRBmin powRBmax
    
    set clipbox $powPlotParam(clipbox${img},$gn)
    if [regexp "clipped" $clipbox] return
    
    catch {image delete ${img}disp$gn}
    if $powPseudoImages {
	image create pict ${img}disp$gn
    } else {
	image create photo ${img}disp$gn
    }
    
    set width  [image width $img]
    set height [image height $img]

    #collect up the inputs to ship to Tk_(Pict||Photo)PutScaledBlock
    set x0 [lindex $clipbox 2]
    set y0 [lindex $clipbox 3]
    set x1 [lindex $clipbox 4]
    set y1 [lindex $clipbox 5]
    foreach {X0 Y0} [powPixelToCanvas $gn $img -0.5 -0.5 $canvas] {}
    foreach {X1 Y1} [powPixelToCanvas $gn $img \
	    [expr $width-0.5] [expr $height-0.5] $canvas] {}
    
    set zoomX [expr ($X1-$X0)/$width]
    set zoomY [expr ($Y0-$Y1)/$height]
		     
    set width  [expr int( ($x1 - $x0)*$zoomX + 0.5 )]
    set height [expr int( ($y1 - $y0)*$zoomY + 0.5 )]
		     
    powPutZoomedBlock $img $gn $x0 $y0 $width $height $zoomX $zoomY

    set curr_img ${img}disp$gn;

    if {$powPseudoImages && $canvas == ".pow.pow"} {
	#reset everything if PseudoColor
	set pixmin $powRBmin($curr_img)
	set pixmax $powRBmax($curr_img)
	
	$curr_img colormap $cmap
	invert_cmap_if_flag_set;
	powSetLut current;    
	powSetRange $gn $img $pixmin $pixmax
    }
}



proc powPlotCurves {gn curves {canvas .pow.pow}} {
    global powPlotParam powCurveParam powcursor powResizeMain

    if [regexp "NULL" $curves] return

    .pow configure -cursor watch
    .pow.pow configure -cursor watch
    catch {.pow.scope configure -cursor watch}

#remove "NULL" from curves list if present
    if [regexp "NULL" $powPlotParam(curves,$gn)] {set powPlotParam(curves,$gn) { }}

    foreach current_curve $curves {

#   if curve is already in list, don't plot it
	if [regexp $current_curve $powPlotParam(curves,$gn)] continue

	if { $canvas == ".pow.scope" } {
	    set trueGn [string range $gn 0 [expr [string length $gn]-6] ]
	    foreach opt $powCurveParam(allOpts,powDef) {
		set powCurveParam(${opt}${current_curve},$gn) \
			$powCurveParam(${opt}${current_curve},$trueGn)
	    }
	} else {
	    foreach opt $powCurveParam(allOpts,powDef) {
		if {! [info exists powCurveParam(${opt}${current_curve},$gn)]} {
		    set powCurveParam(${opt}${current_curve},$gn) \
			    $powCurveParam(${opt},powDef)
		}
	    }
	}

	foreach opt $powCurveParam(allOpts,powDef) {
	    set $opt $powCurveParam(${opt}${current_curve},$gn)
	}

	if { $canvas == ".pow.scope" } {
	    # Shrink point size if drawing in the scope window
	    if { $pSize>0 } {
		set pSize [expr $pSize/2+1]
	    }
	}

#   call curve plotting routine
	$canvas create powCurve $current_curve $gn \
		-pointdisplay $pDisp \
		-pointtype $pShape \
		-pointsize $pSize \
		-pointerror $pSizeErr \
		-pointfill $pFill \
		-linedisplay $lDisp \
		-dash $lStyle \
		-width $lWidth \
		-stairstep $lStep \
		-fill $Color \
		-tags "$gn $current_curve$gn" 

#   add name to list of curves
	lappend powPlotParam(curves,$gn) $current_curve
    }
    .pow configure -cursor $powcursor
    .pow.pow configure -cursor $powcursor
    catch {.pow.scope configure -cursor $powcursor}

}

proc powWhereAmI {x y {canvas ".pow.pow"}} {
    set boxes [$canvas find withtag gbox]
    set topbox -1
    set topgraph ""
    foreach graph [powListGraphs] {
	set gbox [$canvas coords ${graph}box]
	if { $gbox == "" } continue
	if { $x >= [lindex $gbox 0] && $x <= [lindex $gbox 2] \
		&& $y >= [lindex $gbox 1]  && $y <= [lindex $gbox 3] } {
	    set order [lsearch $boxes [$canvas find withtag ${graph}box]]
	    if {$order>$topbox} {set topbox $order; set topgraph $graph }
	}
    }
    if {$topbox>=0} {return $topgraph} else {return "NULL"}
}

#  C  routines: CanvasToGraph, GraphToPixel
#               PixelToGraph, GraphToCanvas
# TCL routines: CanvasToPixel, PixelToCanvas

proc powPixelToCanvas {gn img x y {canvas .pow.pow}} {

    set ccoords [powPixelToGraph $img $x $y]
    set rx [lindex $ccoords 0]
    set ry [lindex $ccoords 1]
    set ccoords [powGraphToCanvas $gn $rx $ry $canvas]

#    puts "PixelToCanvas - $x $y $ccoords $gn"
    return $ccoords 
}

proc powCanvasToPixel {gn img x y {canvas .pow.pow}} {
    global powPlotParam

    set ccoords [powCanvasToGraph $gn $x $y $canvas]
    set rx [lindex $ccoords 0]
    set ry [lindex $ccoords 1]
    set ccoords [powGraphToPixel $img $rx $ry]

#    puts "CanvasToPixel - $x $y $ccoords $gn"
    return $ccoords 
}

proc set_tracker_info {x y {canvas ".pow.pow"}} {
    global powPlotParam powTrackText currimg currgn
    global powPseudoImages powSixties

    if {$canvas == ".pow.scope"} {
	set gn ${currgn}scope
    } else {
	set gn $currgn
    }

    if {[regexp -nocase {[a-z0-9]} $currgn]} {
	set cx [$canvas canvasx $x]
	set cy [$canvas canvasy $y]
	if {[powWhereAmI $cx $cy $canvas] == $gn } {
	    set gcoords [powCanvasToGraph $gn $cx $cy $canvas]
	    set powTrackText(rx) [lindex $gcoords 0]
	    set powTrackText(ry) [lindex $gcoords 1]
	} else {
	    set powTrackText(rx) X
	    set powTrackText(ry) X
	}

    }
    

    if {([info exists currimg] && [$canvas find withtag current_img] != "") && ([$canvas find withtag $gn] != "")} {

	set powTrackText(currimg) $currimg 
	
	set icoords [powCanvasToPixel $gn $currimg [$canvas canvasx $x] \
			 [$canvas canvasy $y] $canvas]
	set imgx [expr round([lindex $icoords 0])]
	set imgy [expr round([lindex $icoords 1])]
    

	set width [image width $currimg]
	set height [image height $currimg]
	

	if "($imgx < $width) && ($imgy < $height) && ($imgx >= 0) && ($imgy >= 0) " {
	    set powTrackText(imgx) $imgx
	    set powTrackText(imgy) $imgy
	    set imgz [powGetImageZ $currimg [expr int($imgx)] [expr int($imgy)]]
	    set powTrackText(imgz) $imgz
	} else {
	    set powTrackText(imgx) "X"
	    set powTrackText(imgy) "X"
	    set powTrackText(imgz) "X"
	}
	set powTrackText(zunits) [powGetImageUnits $currimg Z]
    }
    powUpdateTrackVars
}


proc powStretchGraph {gn xfactor yfactor {canvas ".pow.pow"}} {
    global powPlotParam

    powResizeGraph $gn $xfactor $yfactor $canvas
}


proc powMagGraph {gn newxmagstep newymagstep {canvas ".pow.pow"}} {
    global powPlotParam

    set xfactor \
	    [expr double($newxmagstep) / double($powPlotParam(xmagstep,$gn))]
    set yfactor \
	    [expr double($newymagstep) / double($powPlotParam(ymagstep,$gn))]
    powResizeGraph $gn $xfactor $yfactor $canvas
}

proc powResizeGraph {gn xfactor yfactor {canvas ".pow.pow"}} {
#lowlevel routine, don't call this yourself.  Use powMagGraph or powStretchGraph
#all "resizings" of a graph are done here.  Don't you dare
#do them elsewhere or you'll regret it.
    global powPlotParam powfont powcursor powResizeMain
    global currimggn currimg

    foreach el  [array names powPlotParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == $gn } {
	    set $p1 $powPlotParam($p1,$p2)
	}
    }
    
    
    set bbox [$canvas coords ${gn}box]

#    save initial coordinates

    if {$canvas == ".pow.scope"} {
	set ul [list 20 20]
    } else {
	set ul [list [lindex $bbox 0] [lindex $bbox 1]]
    }


    $canvas scale $gn [lindex $bbox 0] [lindex $bbox 1] $xfactor $yfactor


    set powPlotParam(xmagstep,$gn) [expr $xfactor * $powPlotParam(xmagstep,$gn)]
    set powPlotParam(ymagstep,$gn) [expr $yfactor * $powPlotParam(ymagstep,$gn)]

    powSetGraphMagstep $gn $powPlotParam(xmagstep,$gn) \
	    $powPlotParam(ymagstep,$gn) 
    
    if {![regexp "NULL" $images]} {
	foreach img $powPlotParam(images,$gn) {
	    powMagImage $gn $img $canvas
	}
    }
    
    
    if {$canvas == ".pow.pow"} {
	#change font if necessary
	set gbbox [.pow.pow coords ${gn}box]
	set sizeX  [expr [lindex $gbbox 2] - [lindex $gbbox 0]]
	set sizeY [expr [lindex $gbbox 3] - [lindex $gbbox 1]]
	set boxsize [expr {$sizeX < $sizeY} ? $sizeY : $sizeX]
	
	.pow.pow itemconfigure ${gn}text -font $powfont
	
	
	#draw new tick marks and numbers to go with and new labels
	powRedrawTicks $gn $canvas
	
	#make new GraphHandles
	.pow.pow delete ${gn}handles
	.pow.pow delete ${gn}shandle
	powMakeGraphHandles $gn
	powSelectGraph $gn
    }
    
    
#move the graph to the same upper left position as before
    set newbbox [$canvas coords ${gn}box]
    set new_ul [list [lindex $newbbox 0] [lindex $newbbox 1]]

#    powMoveGraphTo $gn [expr int([lindex $ul 0])] \
#                       [expr int([lindex $ul 1])] $canvas

#return the graph to the canvas if the upper left corner is at negative 
#coordinates
    
#    powRestoreGraph $gn $canvas
    
    
    if {$canvas == ".pow.pow" } {
	#Need to redraw the current image box.  This may not be necessary every
	#time, but this is a lot easier than figuring out wether it is necessary
	catch {powSelectImage $currimggn $currimg}
    }
    
    
    
}



proc powRestoreGraph {gn {canvas .pow.pow}} {
    set bbox [$canvas bbox $gn]
    set x [lindex $bbox 0]
    set y [lindex $bbox 1]
    set mx [expr ($x < 20) ? 20 - $x : 0]
    set my [expr ($y < 20) ? 20 - $y : 0]
    powMoveGraph $gn $mx $my $canvas
}
    


proc tagXdim {can tag} {
    set bbox [$can coords $tag]
    return [expr [lindex $bbox 2] - [lindex $bbox 0]]
}

proc tagYdim {can tag} {
    set bbox [$can coords $tag]
    return [expr [lindex $bbox 3] - [lindex $bbox 1]]
}



proc powStretchGraphToSize {gn xdim ydim {canvas ".pow.pow"}} {
#stretches/shrinks graph to fit in xdim/ydim size
    global  powPlotParam 

    set curr_xdim [tagXdim $canvas ${gn}box]
    set curr_ydim [tagYdim $canvas ${gn}box]

    set xfactor [expr double($xdim)/double($curr_xdim)]
    set yfactor [expr double($ydim)/double($curr_ydim)]
	
    powStretchGraph $gn $xfactor $yfactor $canvas
    
    #save requested current size of graph
    set powPlotParam(xdimdisp,$gn) $xdim
    set powPlotParam(ydimdisp,$gn) $ydim
    
    if {$canvas == ".pow.pow"} {
	powSelectGraph $gn
    }

}


#Plotting routines below here ...
proc powMoveGraph {gn xDist yDist {canvas ".pow.pow"}} {
    global powPlotParam 

    if { $xDist==0 && $yDist==0 } return
    $canvas move $gn $xDist $yDist
    $canvas move ${gn}handles $xDist $yDist
    $canvas move ${gn}yhandle $xDist $yDist
    incr powPlotParam(xo,$gn) $xDist
    incr powPlotParam(yo,$gn) $yDist
}

proc powMoveGraphTo {gn x y {canvas ".pow.pow"}} {
    global powPlotParam 
    set bbox [$canvas coords ${gn}box]
    set xDist [expr $x - [lindex $bbox 0]]
    set yDist [expr $y - [lindex $bbox 1]]

    $canvas move $gn $xDist $yDist
    $canvas move ${gn}handles $xDist $yDist
    $canvas move ${gn}yhandle $xDist $yDist
    set powPlotParam(xo,$gn) $x
    set powPlotParam(yo,$gn) $y
    powRestoreGraph $gn
    
}


proc powMakeGraphHandles {gn} {
    global powPlotParam powfont env
    
#    update idletasks
    set bbox [.pow.pow bbox $gn]
    set left [lindex $bbox 0]
    set top [lindex $bbox 1]
    set right [lindex $bbox 2 ]
    set bot [lindex $bbox 3]

#Make "handle" for graph,  you can pick up the graph and 
#move it around by dragging this
    if [string match "*t*" $powPlotParam(handleposition,$gn)] {
	set y $top
    } elseif [string match "*b*" $powPlotParam(handleposition,$gn)] {
	set y $bot
    } else {
	set y [expr ($top + $bot)/2.0]
    }

    if [string match "*l*" $powPlotParam(handleposition,$gn)] {
	set x $left
    } elseif [string match "*r*" $powPlotParam(handleposition,$gn)] {
	set x $right
    } else {
	set x [expr ($left + $right)/2.0]
    }

# .pow.ms${gn}handle - the 'ms' stands for Move/select and is necessary
# to allow graph names to start with a capital 

    if [winfo exists .pow.ms[powCleanName ${gn}]handle] {
	.pow.ms[powCleanName ${gn}]handle configure -bg $powPlotParam(bgcolor,$gn) \
	    -text $powPlotParam(handletext,$gn) -cursor fleur 
    } else {
	button .pow.ms[powCleanName ${gn}]handle \
		-bg $powPlotParam(bgcolor,$gn) \
		-text $powPlotParam(handletext,$gn) -cursor fleur 
	bind .pow.ms[powCleanName ${gn}]handle <ButtonPress-1> \
		"set powMoveX %X ; set powMoveY %Y"
	bind .pow.ms[powCleanName ${gn}]handle <B1-Motion> \
		"powHideCurves $gn ; powMoveHandle $gn %X %Y"
	bind .pow.ms[powCleanName ${gn}]handle <ButtonRelease-1> \
	    "+powShowCurves $gn; powSelectGraph $gn; \
             powReconfigureToplevel \$powResizeMain"
#the next binding lets us get the graph back if we drag it off the viewable
#area
#	bind .pow <ButtonRelease-1> {
#	    powReconfigureToplevel $powResizeMain
#	}

    }

    raise .pow.ms[powCleanName ${gn}]handle .pow.pow

  

    .pow.pow create window $x $y \
	-tags "${gn}handle handle ghandle ${gn}handles canvas_window" \
        -anchor $powPlotParam(handleanchor,$gn) \
	-window .pow.ms[powCleanName ${gn}]handle

#Make "stretch handle" for graph,  you will be able to expand the graph
#by dragging this.
    if [winfo exists .pow.s[powCleanName ${gn}]handle] {
	.pow.s[powCleanName ${gn}]handle configure -bg $powPlotParam(bgcolor,$gn) \
	    -bitmap stretcharrow\
	    -cursor bottom_right_corner
    } else {
	button .pow.s[powCleanName ${gn}]handle -bg $powPlotParam(bgcolor,$gn) \
	    -bitmap stretcharrow \
	    -cursor bottom_right_corner
	bind .pow.s[powCleanName ${gn}]handle <B1-Motion> "powStretch $gn %X %Y; "
	bind .pow.s[powCleanName ${gn}]handle <ButtonPress-1> \
	    "powBeginStretch $gn; set fixedStretch \"yes\"; powStretch $gn %X %Y; "
	bind .pow.s[powCleanName ${gn}]handle <Shift-ButtonPress-1> \
	    "powBeginStretch $gn; set fixedStretch \"no\"; powStretch $gn %X %Y; "
	bind .pow.s[powCleanName ${gn}]handle <ButtonRelease-1> \
	    "powEndStretch $gn "
    }


    .pow.pow create window $right $bot\
	-tags " ${gn}shandle shandle handle ${gn}handles canvas_window" \
	-anchor se -window .pow.s[powCleanName ${gn}]handle
   

    raise .pow.s[powCleanName ${gn}]handle .pow.pow


#Make a colored background
    .pow.pow create rectangle $bbox \
	-tags "${gn}handles ${gn}bkg graphbkg" \
	-fill $powPlotParam(bgcolor,$gn) -outline $powPlotParam(bgcolor,$gn)
    .pow.pow lower ${gn}bkg

#Store position of "Select" handle relative to graph box
#    update idletasks
    set hcoords [.pow.pow coords ${gn}handle]
    set bcoords [.pow.pow coords ${gn}box]
    set powPlotParam(handleoffsetx,$gn) \
	[expr [lindex $hcoords 0] - [lindex $bcoords 0]]
    set powPlotParam(handleoffsety,$gn) \
	[expr [lindex $hcoords 1] - [lindex $bcoords 1]]
}

proc powCleanName {gn} {
    regsub -all {\.} $gn {_} a
    return $a
}


proc powHideCurves { gn } {
    global powPlotParam

    foreach crv $powPlotParam(curves,$gn) {
	if {$crv=="NULL"} continue
	.pow.pow itemconfig ${crv}${gn} -hidden 1
    }
}

proc powShowCurves { gn } {
    global powPlotParam

    foreach crv $powPlotParam(curves,$gn) {
	if {$crv=="NULL"} continue
	.pow.pow itemconfig ${crv}${gn} -hidden 0
    }
}

proc powMoveHandle {gn x y} {
    global powMoveX powMoveY powPlotParam

    # Calculate root bounding box of .pow.pow canvas (- a little)

    set left  [expr [winfo rootx  .pow.pow]         + 10]
    set top   [expr [winfo rooty  .pow.pow]         + 10]
    set right [expr [winfo width  .pow.pow] + $left - 20]
    set bott  [expr [winfo height .pow.pow] + $top  - 20]
    
    # Check whether we have moved outside of the .pow.pow canvas

    if { $x < $left } {
	set x $left
    } elseif { $x > $right } {
	set x $right
    }

    if { $y < $top } {
	set y $top
    } elseif { $y > $bott } {
	set y $bott
    }

    set dx [expr $x - $powMoveX]
    set dy [expr $y - $powMoveY]

    powMoveGraph $gn [expr $x - $powMoveX] [expr $y - $powMoveY]

    set powMoveX $x 
    set powMoveY $y
}


proc powFinishGraph {gn {canvas ".pow.pow"}} {
    global powPlotParam curX curY powcursor powbg  powFirstTimeThroughFlag
    global powResizeMain powGUI
    
    if {$canvas == ".pow.pow"} {
	
	.pow.pow delete ${gn}handles
	.pow.pow delete ${gn}shandle
	powMakeGraphHandles $gn

#	.pow.pow bind ${gn}handle <Enter> ".pow.pow itemconfigure ${gn}handle -fill red"
#	.pow.pow bind ${gn}handle <Leave> ".pow.pow itemconfigure ${gn}handle -fill purple"
#	.pow.pow bind ${gn}handle <ButtonPress-1> "set curX %x; set curY %y;  .pow.pow configure -cursor fleur;"
#	.pow.pow bind ${gn}handle <ButtonRelease-1> ".pow.pow configure -cursor \$powcursor; powRestoreGraph $gn;  powReconfigureToplevel \$powResizeMain; powSelectGraph $gn"
	
	

	

	if {$powFirstTimeThroughFlag} {
	    powReconfigureToplevel 1
	    set powFirstTimeThroughFlag 0
	} else {
	    powReconfigureToplevel $powResizeMain
	}

# Force redrawing of scope box, in case there is new graph information
	if $powGUI {
# Lets not
#	    .pow.scope delete all
	}

	powSelectGraph $gn
	powRedrawRegions
    }

    .pow configure -cursor $powcursor
    .pow.pow configure -cursor $powcursor
    catch {.pow.scope configure -cursor $powcursor}
}


proc powBeginROI {x y {canvas .pow.pow}} {
    global roi_xo roi_yo saveROI

    if {[$canvas find withtag ROI] != ""} {
	set saveROI [$canvas coords ROI]
    }

    set roi_xo [$canvas canvasx $x]
    set roi_yo [$canvas canvasy $y]
    $canvas create rectangle $x $y $x $y -tags ROI -outline blue
}

proc powDragROI {x y {canvas .pow.pow}} {
    global roi_xo roi_yo
    $canvas delete ROI
    $canvas create rectangle $roi_xo $roi_yo \
	[$canvas canvasx $x] [$canvas canvasy $y] \
	-tags ROI -outline blue
}

proc powPanROI {x y {canvas .pow.pow}} {
    global roi_xo roi_yo
    set ROIbbox [$canvas coords ROI]
    set halfwidth [expr ([lindex $ROIbbox 2] - [lindex $ROIbbox 0])/2.0]
    set halfheight [expr ([lindex $ROIbbox 3] - [lindex $ROIbbox 1])/2.0]
    $canvas delete ROI
    $canvas create rectangle [expr $x - $halfwidth] [expr $y - $halfheight] [expr $x + $halfwidth] [expr $y + $halfheight] -tags ROI -outline blue
}


proc powEndROI { zoomback {canvas .pow.pow}} {
global saveROI powGUI
#if zoomback is true, we're restoring the "default size" of the graph
#otherwise, this is the end of a user dragging an ROI box
    global powPlotParam currgn powfont axisToChainHash chainToAxisHash powResizeMain
    global powGraphsTagRangeList powTagsColorMap currimg
    global powGraphsTagRectList

    if {$canvas == ".pow.scope"} {
        set currgraph  ${currgn}scope
    } else {
	set currgraph  $currgn
    }

    if {!($zoomback)} {
	set ROIbbox [$canvas coords ROI]
	set x0 [lindex $ROIbbox 0]
	set x1 [lindex $ROIbbox 2]
	set y0 [lindex $ROIbbox 3]
	set y1 [lindex $ROIbbox 1]

#return if ROI is <= 1 screen pixel (they probably clicked without moving)
	if { abs($x0 -$x1) <= 1 || abs($y0 - $y1) <= 1} {
	    $canvas delete ROI
	    if [info exists saveROI] {
#redraw original ROI
		$canvas create rectangle $saveROI -tags ROI -outline blue
	    }
	    return
	}
#catch bboxes smaller than 4 pixels
	if {[info exists currimg]} {
	    set llpix [powCanvasToPixel $currgraph $currimg $x0 $y0 $canvas]
	    set llpx [lindex $llpix 0]
	    set llpy [lindex $llpix 1]
	    set urpix [powCanvasToPixel $currgraph $currimg $x1 $y1 $canvas]
	    set urpx [lindex $urpix 0]
	    set urpy [lindex $urpix 1]
	    

	    if {(abs($urpx - $llpx) < 2) || (abs($urpy - $llpy) < 2)} {
		set newcan \
		    [powPixelToCanvas $currgraph $currimg [expr $urpx + 2] [expr $urpy + 2] $canvas]
		set x1 [lindex $newcan 0]
		set y1 [lindex $newcan 1]
	    } 
	}
	


#get "real" coordinates of ROIbbox
	set gcoords  [powCanvasToGraph $currgraph $x0 $y0 $canvas] 
	set llx [lindex $gcoords 0]
	set lly [lindex $gcoords 1]
	set gcoords  [powCanvasToGraph $currgraph $x1 $y1 $canvas] 
	set urx [lindex $gcoords 0]
	set ury [lindex $gcoords 1]
	
    }

    set graphlist $currgn

    if {[array names axisToChainHash ${currgn}X] != ""} {
	set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${currgn}X))]
    }

    if {[array names axisToChainHash ${currgn}Y] != ""} {
	set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${currgn}Y))]
    }

    set principal 1
    
    foreach graph [concat $graphlist] {
	if {!$principal} {
		set axis [chopped $graph]
		set graph [chop $graph]
	}



	if {!$zoomback} {
	    if $principal {
#note rROIbbox is in "scientific" coordinate, other bboxs are in "X" coordinates (i.e. upper left origin)
		set rROIbbox [list $llx $lly $urx $ury]
		if { $canvas != ".pow.scope" } { $canvas delete ROI }
	    } else {
		if {$axis == "X"} {
		    set abox [.pow.pow coords ${graph}box]
		    set cllx $llx
		    set clly [lindex [powCanvasToGraph $graph \
			     [lindex $abox 0] [lindex $abox 3] .pow.pow] 1]
		    set curx $urx
		    set cury [lindex [powCanvasToGraph $graph \
                             [lindex $abox 2] [lindex $abox 1] .pow.pow] 1]
		    set rROIbbox [list $cllx $clly $curx $cury]
		} else {
		    set abox [.pow.pow coords ${graph}box]
		    set cllx [lindex [powCanvasToGraph $graph \
                             [lindex $abox 0] [lindex $abox 3] .pow.pow] 0]
		    set clly $lly
		    set curx [lindex [powCanvasToGraph $graph \
                             [lindex $abox 2] [lindex $abox 1] .pow.pow] 0]
		    set cury $ury
		    set rROIbbox [list $cllx $clly $curx $cury]
		}	    
	    }
	}

    
#get together everything you need for the next call to powCreateGraph
	set graph_position [.pow.pow coords ${graph}handle]
	set ROIcurves $powPlotParam(curves,$graph)
	set ROIimages $powPlotParam(images,$graph)
	set ROIunits [list $powPlotParam(xunits,$graph) \
		           $powPlotParam(yunits,$graph) \
			   $powPlotParam(xlabel,$graph) \
			   $powPlotParam(ylabel,$graph) ]

	set ROIgraphOptions [powGetGraphOptions $graph]


# Make new graph
	powUnmapGraph $graph 0
	if {$zoomback} {

	    eval [concat powCreateGraph $graph \{$ROIcurves\} \{$ROIimages\} \
                                    $ROIunits $powPlotParam(xdimdisp,$graph) \
		                    $powPlotParam(ydimdisp,$graph)]
	    set rROIbbox [list $powPlotParam(xBot,$graph) \
                               $powPlotParam(yBot,$graph) \
                               $powPlotParam(xTop,$graph) \
                               $powPlotParam(yTop,$graph) ]
	    set powPlotParam(zoomed,$graph) 0

	} else {

	    eval [concat powCreateGraph $graph \{$ROIcurves\} \{$ROIimages\} \
		                    $ROIunits $powPlotParam(xdimdisp,$graph) \
		                    $powPlotParam(ydimdisp,$graph) $rROIbbox]
	    set powPlotParam(zoomed,$graph) 1
	}

	eval [concat powGraphOptions $graph $ROIgraphOptions]
	set principal 0
    }

    if {$powGUI && $canvas != ".pow.scope"} {
	.pow.scope delete ROI
	powDrawScopeROI $rROIbbox
    }

    if {[info exists powGraphsTagRangeList($currgn)]} {
	foreach tagrangelist [concat $powGraphsTagRangeList($currgn)] {
	    set tag [lindex $tagrangelist 3]
	    eval [concat powColorRange $currgn $tagrangelist $powTagsColorMap($tag) 1]
	}
    }
    if {[info exists powGraphsTagRectList($currgn)]} {
	foreach tagrectlist [concat $powGraphsTagRectList($currgn)] {
	    set tag [lindex $tagrectlist 4]
	    eval [concat powColorRect $currgn $tagrectlist $powTagsColorMap($tag) 1]
	}
    }

}


proc powDrawScopeROI {rROIbbox } {
    global currgn
    set gn ${currgn}scope
    set rllx [lindex $rROIbbox 0]
    set rlly [lindex $rROIbbox 1]
    set rurx [lindex $rROIbbox 2]
    set rury [lindex $rROIbbox 3]
    set ccoords [powGraphToCanvas $gn $rllx $rlly .pow.scope]
    set ulx [lindex $ccoords 0]
    set lry [lindex $ccoords 1]
    set ccoords [powGraphToCanvas $gn $rurx $rury .pow.scope]
    set lrx [lindex $ccoords 0]
    set uly [lindex $ccoords 1]
    .pow.scope create rectangle $ulx $uly $lrx $lry -tags ROI -outline blue
}




proc powStretch {gn x y} {
    global stretch_xo stretch_yo stretch_powx stretch_powy
    global powPlotParam powfont stretch_xmarg stretch_ymarg
    global new_xdim new_ydim fixedStretch

    set x [.pow.pow canvasx [expr $x - $stretch_powx]]
    set y [.pow.pow canvasy [expr $y - $stretch_powy]]
    .pow.pow delete current_gn

#check magstep
    set new_xdim [expr $x - $stretch_xo - $stretch_xmarg]
    set new_ydim [expr $y - $stretch_yo - $stretch_ymarg]
    set curr_xdim [tagXdim .pow.pow ${gn}box]
    set curr_ydim [tagYdim .pow.pow ${gn}box]
    if {$new_xdim > 2 && $new_ydim > 2} {
	set xfactor [expr double($new_xdim)/double($curr_xdim)]
	set yfactor [expr double($new_ydim)/double($curr_ydim)]
	if { $fixedStretch == "yes" } {
	    if { $xfactor < $yfactor } {
		set yfactor $xfactor
	    } else {
		set xfactor $yfactor
	    }
	}
	set new_xdim [expr $xfactor*$curr_xdim]
	set new_ydim [expr $yfactor*$curr_ydim]
    } else { 
	set new_xdim $curr_xdim
	set new_ydim $curr_ydim
    }
#make magstep label   
    set sizeText [format "%4d x %4d" [expr round($new_xdim)] \
                                     [expr round($new_ydim)]  ]
    .pow.ms[powCleanName ${gn}]handle configure -text "GraphSize: $sizeText" 
    set x [expr $new_xdim+$stretch_xo+$stretch_xmarg]
    set y [expr $new_ydim+$stretch_yo+$stretch_ymarg]
    .pow.pow create rectangle  $stretch_xo $stretch_yo $x $y \
	-tags  "current_gn ${gn}yhandle handle ohandle" -outline yellow -width 3
    .pow.pow coords ${gn}shandle $x $y
}

proc powBeginStretch {gn} {
    global stretch_xo stretch_yo stretch_powx stretch_powy powbg
    global powPlotParam stretch_xmarg stretch_ymarg

    powSelectGraph $gn
    set bbox [.pow.pow coords current_gn]
    set stretch_xo [lindex $bbox 0]
    set stretch_yo [lindex $bbox 1]
    set stretch_powx [winfo rootx .pow.pow]
    set stretch_powy [winfo rooty .pow.pow]
    set gbox [.pow.pow coords ${gn}box]
    set stretch_xmarg [expr ([lindex $bbox 2]-[lindex $bbox 0]) \
	                  - ([lindex $gbox 2]-[lindex $gbox 0]) ]
    set stretch_ymarg [expr ([lindex $bbox 3]-[lindex $bbox 1]) \
	                  - ([lindex $gbox 3]-[lindex $gbox 1]) ]

}


proc powEndStretch {gn} {
    global powcursor powResizeMain powPlotParam new_xdim new_ydim
    global fixedStretch

    powStretchGraphToSize $gn $new_xdim $new_ydim
    powSelectGraph $gn
    powReconfigureToplevel $powResizeMain
}


proc powStartNewRow { } {
    global powOpenAreaTop
 
#    update idletasks
    set powOpenAreaTop [lindex [.pow.pow bbox all] 3]
}

proc powInitGraph {gn xMin xMax yMin yMax xunits yunits xLabel yLabel \
		       {canvas ".pow.pow"} \
		       {xDim 600} {yDim 400} \
		       {xDimDisp 600} {yDimDisp 400} {aspect yes} \
		       {xmargin 60} {ymargin 60} } {

# An array of plotting parameters
 global powPlotParam powOpenAreaTop powbg
 global powGrid powGridColor powGridDash powNumTicks

#puts "gn $gn xMin $xMin xMax $xMax yMin $yMin yMax $yMax xunits $xunits yunits $yunits xLabel $xLabel yLabel $yLabel canvas $canvas xDim $xDim yDim $yDim xpixunit $xpixunit ypixunit $ypixunit aspect $aspect"

 .pow configure -cursor watch
 .pow.pow configure -cursor watch
 catch {.pow.scope configure -cursor watch}

#puts "xmin xmax ymin ymax: $xMin $xMax $yMin $yMax"



#####################Plot
set powPlotParam(images,$gn) "NULL"
set powPlotParam(curves,$gn) "NULL"
set powPlotParam(zoomed,$gn) 0
 if {![info exists powOpenAreaTop]} {set powOpenAreaTop 10}
 if {[info exists powPlotParam(xo,$gn)]} {
#   Do nothing, thereby keeping the xo/yo values intact
 } elseif {$canvas == ".pow.pow"} then {
#     update idletasks
     set bbox [.pow.pow bbox all]
     if {$bbox != ""} {
	 set leftSide [lindex $bbox 0]
	 .pow.pow addtag currentRow enclosed $leftSide $powOpenAreaTop \
	     [lindex $bbox 2]  [lindex $bbox 3]
#	 update idletasks
	 set bbox [.pow.pow bbox currentRow]
	 .pow.pow dtag currentRow
	 if {$bbox != ""} then {
	     set powPlotParam(xo,$gn) [expr [lindex $bbox 2] + $xmargin]
	 } else {
	     set powPlotParam(xo,$gn) [expr $leftSide + $xmargin]
	 } 
     } else {
	 set powPlotParam(xo,$gn) $xmargin
     }
     set powPlotParam(yo,$gn) [expr $powOpenAreaTop  + $ymargin ]
 } else {
     set powPlotParam(xo,$gn) 20
     set powPlotParam(yo,$gn) 20
 }
 set powPlotParam(graphHeight,$gn) $yDim
 set powPlotParam(graphWidth,$gn) $xDim
 set powPlotParam(xdimdisp,$gn) $xDimDisp
 set powPlotParam(ydimdisp,$gn) $yDimDisp
 set powPlotParam(xBot,$gn) $xMin
 set powPlotParam(xTop,$gn) $xMax
 set powPlotParam(xunits,$gn) $xunits
 set powPlotParam(xlabel,$gn) $xLabel
 if [regexp -nocase NULL $xunits] {
     set powPlotParam(xString,$gn) "$xLabel"     
 } else {
     set powPlotParam(xString,$gn) "$xLabel ($xunits)"     
 }
 set powPlotParam(yBot,$gn) $yMin
 set powPlotParam(yTop,$gn) $yMax 
 set powPlotParam(yunits,$gn) $yunits
 set powPlotParam(ylabel,$gn) $yLabel
 if [regexp -nocase NULL $yunits] {
     set powPlotParam(yString,$gn) "$yLabel"     
 } else {
     set powPlotParam(yString,$gn) "$yLabel ($yunits)"     
 }
 
###################defaults for optional graph params handled by
#                  powGraphOptions
if {![info exists powPlotParam(bgcolor,$gn)]} {
    set powPlotParam(bgcolor,$gn) $powbg
}
if {![info exists powPlotParam(xmargin,$gn)]} {
    set powPlotParam(xmargin,$gn) $xmargin
}
if {![info exists powPlotParam(ymargin,$gn)]} {
    set powPlotParam(ymargin,$gn) $ymargin
}
if {![info exists powPlotParam(handletext,$gn)]} {
    set powPlotParam(handletext,$gn) "Select/Move: $gn"
}
if {![info exists powPlotParam(handleanchor,$gn)]} {
    set powPlotParam(handleanchor,$gn) "sw"
}
if {![info exists powPlotParam(handleposition,$gn)]} {
    set powPlotParam(handleposition,$gn) "tl"
}
if {![info exists powPlotParam(Grid,$gn)]} {
    set powPlotParam(Grid,$gn) $powGrid
}
if {![info exists powPlotParam(GridColor,$gn)]} {
    set powPlotParam(GridColor,$gn) $powGridColor
}
if {![info exists powPlotParam(GridDash,$gn)]} {
    set powPlotParam(GridDash,$gn) $powGridDash
}
if {![info exists powPlotParam(NumTicks,$gn)]} {
    set powPlotParam(NumTicks,$gn) $powNumTicks
}
if {![info exists powPlotParam(FixedAspect,$gn)]} {
    set powPlotParam(FixedAspect,$gn) $aspect
}

set xmagstep  [expr double($xDimDisp)/$xDim]
set ymagstep  [expr double($yDimDisp)/$yDim]
set newaspect [expr $xmagstep/$ymagstep]

if { $powPlotParam(FixedAspect,$gn)=="yes"} {
    if { [info exists powPlotParam(xmagstep,$gn)] } {
	set aspect [expr $powPlotParam(xmagstep,$gn) \
		         / $powPlotParam(ymagstep,$gn) ]
	if { $newaspect > $aspect } {
	    set xmagstep [expr $ymagstep*$aspect]
	} else {
	    set ymagstep [expr $xmagstep/$aspect]
	}
    } else {
	if { $xmagstep<$ymagstep } {
	    set ymagstep $xmagstep
	} else {
	    set xmagstep $ymagstep
	}
    }
}

set powPlotParam(xmagstep,$gn) $xmagstep
set powPlotParam(ymagstep,$gn) $ymagstep
powSetGraphMagstep $gn $xmagstep $ymagstep


xselPlot $gn $canvas


if {$canvas == ".pow.pow"} {
# Can't have an image from the previous graph interfering with a new one
    powDeSelectImage
    powAdornGraph $gn $canvas
}        
}

proc xselPlot {gn {canvas .pow.pow}} {
    global powPlotParam currimg


# clean the canvas if there was a previous version of this graph
    $canvas delete $gn


    foreach el  [array names powPlotParam]  {
      set p1 [lindex [split $el ,] 0]
      set p2 [lindex [split $el ,] 1]
	if { $p2 == $gn } {
           set $p1 $powPlotParam($p1,$p2)
        }
    }

# plot a guess at the box. This is done to define a lower left
# corner on the canvas to refer to.
    
    $canvas  create rectangle $xo $yo \
               [expr $graphWidth*$xmagstep + $xo]  \
               [expr $graphHeight*$ymagstep + $yo]  \
	       -tags "$gn ${gn}box ${gn}line gbox" -outline black
}


proc powRedrawTicks {gn {canvas .pow.pow}} {
#since calculating the tickmarks is so time consuming, we need 
#a way to simply "redraw" existing tickmarks 
    global powfont powPlotParam  currgn

    set bbox [$canvas coords ${gn}box]
    set x0 [lindex $bbox 0]
    set x1 [lindex $bbox 2]
    set y0 [lindex $bbox 1]
    set y1 [lindex $bbox 3]

    foreach lfttick [$canvas find withtag ${gn}lftticks] {
	set coords [$canvas coords $lfttick]
	set tx0 [expr $x0 - 10]
	set tx1 $x0
	set ty0 [lindex $coords 1]
	set ttags [$canvas itemcget $lfttick -tags]
	$canvas delete $lfttick
	$canvas create line $tx0 $ty0 $tx1 $ty0 -tags $ttags -fill black
    }
    foreach rgttick [$canvas find withtag ${gn}rgtticks] {
	set coords [$canvas coords $rgttick]
	set tx0 $x1
	set tx1 [expr $x1 + 10]
	set ty0 [lindex $coords 1]
	set ttags [$canvas itemcget $rgttick -tags]
	$canvas delete $rgttick
	$canvas create line $tx0 $ty0 $tx1 $ty0 -tags $ttags -fill black
    }
    foreach toptick [$canvas find withtag ${gn}topticks] {
	set coords [$canvas coords $toptick]
	set tx0 [lindex $coords 0]
	set ty0 [expr $y0 - 10]
	set ty1 $y0
	set ttags [$canvas itemcget $toptick -tags]
	$canvas delete $toptick
	$canvas create line $tx0 $ty0 $tx0 $ty1 -tags $ttags -fill black
    }
    foreach bottick [$canvas find withtag ${gn}botticks] {
	set coords [$canvas coords $bottick]
	set tx0 [lindex $coords 0]
	set ty0 [expr $y1 + 10]
	set ty1 $y1
	set ttags [$canvas itemcget $bottick -tags]
	$canvas delete $bottick
	$canvas create line $tx0 $ty0 $tx0 $ty1 -tags $ttags -fill black
    }
    foreach lftnum [$canvas find withtag ${gn}lftnums] {
	set coords [$canvas coords $lftnum]
	set tx0 [expr $x0 - 15]
	set ty0 [lindex $coords 1]
	set ttags [$canvas itemcget $lftnum -tags]
	set ttext [$canvas itemcget $lftnum -text]
	$canvas delete $lftnum
	$canvas create text $tx0 $ty0 -tags $ttags -font $powfont \
		    -text $ttext -anchor e
    }
    foreach botnum [$canvas find withtag ${gn}botnums] {
	set coords [$canvas coords $botnum]
	set tx0 [lindex $coords 0]
	set ty0 [expr $y1 + 15]
	set ttags [$canvas itemcget $botnum -tags]
	set ttext [$canvas itemcget $botnum -text]
	$canvas delete $botnum
	$canvas create text $tx0 $ty0 -tags $ttags -font $powfont \
	    -text $ttext -anchor n
    }

#redraw X and Y labels
#    set xString [$canvas itemcget ${gn}xlabel -text]
#    set yString [$canvas itemcget ${gn}ylabel -text]
    set xString $powPlotParam(xString,$gn)
    set yString $powPlotParam(yString,$gn)
    $canvas delete ${gn}xlabel
    $canvas delete ${gn}ylabel

    if [regexp {[^ ]} $xString] {
	.pow.pow create text [expr ($x0 + $x1)/2 ] \
	    [expr $y1 + 35] -text $xString -anchor n \
	    -tags "$gn ${gn}label ${gn}xlabel ${gn}text" -font $powfont
    }


    if [regexp {[^ ]} $yString] {
	.pow.pow create text [expr $x0 - $powPlotParam(xmargin,$gn)/2] \
	    [expr $y0 - 5] -text $yString -anchor sw -justify left \
	    -tags "$gn ${gn}label ${gn}ylabel ${gn}text" -font $powfont
    }


#eschew overlappage
#this was causing some godawful shrinking problems, so take it out.

#    foreach lftnum [$canvas find withtag ${gn}lftnums] {
#	while {[powOverlapTest $lftnum $canvas]}  {
#	    $canvas move $lftnum -5 0
#	}
#    }
#    foreach botnum [$canvas find withtag ${gn}botnums] {
#	while {[powOverlapTest $botnum $canvas]}  {
#	    $canvas move $botnum 0 +5
#	}
#    }
}


proc powOverlapTest {id {canvas .pow.pow}} {
    set bb [$canvas bbox $id]
    if {$bb != ""} {
	set olap [eval $canvas find overlapping $bb]
    } else {
	return 0
    }
    foreach item $olap {
	set tags [$canvas gettags $item]
	if {$item != $id && !([string match "*handle*" $tags ])} {
	    return 1
	}
    }
    return 0
}


proc powRedrawBox {gn {canvas .pow.pow}} {
    global powPlotParam currimg
    
    foreach el  [array names powPlotParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == $gn } {
	    set $p1 $powPlotParam($p1,$p2)
        }
    }
    
#find corners of new box
    set x0 [lindex [$canvas coords ${gn}box] 0]
    set y0 [lindex [$canvas coords ${gn}box] 3]
    set ccoords [powGraphToCanvas $gn $xTop $yTop $canvas] 
    set x1 [lindex $ccoords 0]
    set y1 [lindex $ccoords 1]
    
#remove previous box    
    $canvas delete ${gn}box

# plot the new box
    
    $canvas  create rectangle $x0 $y1 $x1 $y0 \
	    -tags "$gn gbox ${gn}box ${gn}line" -outline black

}


proc powChangeGrid { {redraw 0} } {
    global powPlotParam currgn powGrid powGridColor powGridDash powNumTicks

    if {![info exists currgn]} {return}

    set powPlotParam(GridColor,$currgn) $powGridColor
    set powPlotParam(GridDash,$currgn) $powGridDash
    set powPlotParam(NumTicks,$currgn) $powNumTicks

    if {$redraw} {
	set powPlotParam(Grid,$currgn) $powGrid
	powAdornGraph $currgn .pow.pow
    } else {
	.pow.pow itemconfig ${currgn}grid -fill $powGridColor -dash $powGridDash
    }
}

proc powContour { } {
    global currimg curr_img powRBmin powRBmax powbg powContourParam currgn

    if { ![info exists currimg] || $currimg=="NULL" || $currimg=="" } {
	tk_messageBox -message "Select a graph with an image first." \
		-title "No Image" -type ok -parent .pow
	return
    }

    set powContourParam(image)    $currimg
    set powContourParam(gn)       $currgn
    set powContourParam(res)      2
    set powContourParam(separate) no
    set powContourParam(nContrs)  10
    set lst [powGetTics $powRBmin($currimg) $powRBmax($currimg) 10 0]
    set powContourParam(min)      [lindex $lst 0]
    set powContourParam(max)      [lindex $lst end]
    set powContourParam(scale)    linear

    if {[winfo exists .powcontour]} {destroy .powcontour}
    powToplevel .powcontour .pow "-bg $powbg"
    catch {wm title .powcontour "Create Contours"}

    button .powcontour.help -text "Help" \
	    -command {powHelp Contours.html} \
	    -bg $powbg -takefocus 0
    
    label .powcontour.image -bg $powbg -text "Image:"
    label .powcontour.currimg -bg yellow -fg black -text $currimg

    label .powcontour.imgrng -bg $powbg -text "Image Range:"
    frame .powcontour.imgfrm -bg $powbg
    label .powcontour.imgfrm.min -bg $powbg -width 10 \
	    -text "$powRBmin($currimg)"
    label .powcontour.imgfrm.dash -bg $powbg -text " - "
    label .powcontour.imgfrm.max -bg $powbg -width 10 \
	    -text "$powRBmax($currimg)"
    pack .powcontour.imgfrm.min -in .powcontour.imgfrm -side left \
	    -padx 4 -pady 1 -fill x -expand 1
    pack .powcontour.imgfrm.dash -in .powcontour.imgfrm -side left \
	    -padx 4 -pady 1
    pack .powcontour.imgfrm.max -in .powcontour.imgfrm -side left \
	    -padx 4 -pady 1 -fill x -expand 1

    label .powcontour.pixrng -bg $powbg -text "Contour Range:"
    frame .powcontour.pixfrm -bg $powbg
    entry .powcontour.pixfrm.min -bg $powbg -width 10 \
	    -textvariable powContourParam(min) -takefocus 1
    label .powcontour.pixfrm.dash -bg $powbg -text " - "
    entry .powcontour.pixfrm.max -bg $powbg -width 10 \
	    -textvariable powContourParam(max) -takefocus 1
    pack .powcontour.pixfrm.min -in .powcontour.pixfrm -side left \
	    -padx 4 -pady 1 -fill x -expand 1
    pack .powcontour.pixfrm.dash -in .powcontour.pixfrm -side left \
	    -padx 4 -pady 1
    pack .powcontour.pixfrm.max -in .powcontour.pixfrm -side left \
	    -padx 4 -pady 1 -fill x -expand 1

    label .powcontour.scale -bg $powbg -text "Scale:"
    frame .powcontour.sclbutt -bg $powbg
    radiobutton .powcontour.sclbutt.linear -bg $powbg -text Linear \
	    -variable powContourParam(scale) -value linear \
	    -highlightthickness 0 -takefocus 0
    radiobutton .powcontour.sclbutt.sqrt -bg $powbg -text Sqrt \
	    -variable powContourParam(scale) -value sqrt \
	    -highlightthickness 0 -takefocus 0
    radiobutton .powcontour.sclbutt.log -bg $powbg -text Log \
	    -variable powContourParam(scale) -value log \
	    -highlightthickness 0 -takefocus 0
    pack .powcontour.sclbutt.linear -in .powcontour.sclbutt -side left \
	    -padx 4 -pady 1
    pack .powcontour.sclbutt.sqrt -in .powcontour.sclbutt -side left \
	    -padx 4 -pady 1
    pack .powcontour.sclbutt.log -in .powcontour.sclbutt -side left \
	    -padx 4 -pady 1

    label .powcontour.ncntrs -bg $powbg -text "# Contours:"
    frame .powcontour.ncntrsbutt -bg $powbg
    button .powcontour.ncntrsbutt.less -bg $powbg -text "<" \
	    -command { incr powContourParam(nContrs) -1 } -takefocus 0
    entry  .powcontour.ncntrsbutt.numb -bg $powbg \
	    -textvariable powContourParam(nContrs) -width 5 -takefocus 1
    button .powcontour.ncntrsbutt.more -bg $powbg -text ">" \
	    -command { incr powContourParam(nContrs) 1 } -takefocus 0
    pack .powcontour.ncntrsbutt.less -in .powcontour.ncntrsbutt -side left \
	    -pady 1
    pack .powcontour.ncntrsbutt.numb -in .powcontour.ncntrsbutt -side left \
	    -pady 1
    pack .powcontour.ncntrsbutt.more -in .powcontour.ncntrsbutt -side left \
	    -pady 1
    trace variable powContourParam(nContrs) w { powSetContours }
    trace variable powContourParam(min) w { powSetContours }
    trace variable powContourParam(max) w { powSetContours }
    trace variable powContourParam(scale) w { powSetContours }
	    
    label .powcontour.clist -bg $powbg -text "Contours:"

    frame .powcontour.cntrs -bg $powbg
    scrollbar .powcontour.cntrs.scrolly -orient vertical -takefocus 0 \
	    -command {.powcontour.cntrs.lst yview} -bg $powbg
    text .powcontour.cntrs.lst -bg $powbg -width 20 -height 5 \
	    -yscrollcommand {.powcontour.cntrs.scrolly set } \
	    -takefocus 0

    grid .powcontour.cntrs.lst     -in .powcontour.cntrs -row 1 -column 1 \
	    -sticky news
    grid .powcontour.cntrs.scrolly -in .powcontour.cntrs -row 1 -column 2 \
	    -sticky news
    grid rowconfigure    .powcontour.cntrs 1 -weight 1
    grid columnconfigure .powcontour.cntrs 1 -weight 1

    label .powcontour.res -bg $powbg -text "Resolution:"
    frame .powcontour.resbutt -bg $powbg
    radiobutton .powcontour.resbutt.high -bg $powbg -text High \
	    -variable powContourParam(res) -value 1 -highlightthickness 0 \
	    -takefocus 0
    radiobutton .powcontour.resbutt.med -bg $powbg -text Medium \
	    -variable powContourParam(res) -value 2 -highlightthickness 0 \
	    -takefocus 0
    radiobutton .powcontour.resbutt.low -bg $powbg -text Low \
	    -variable powContourParam(res) -value 3 -highlightthickness 0 \
	    -takefocus 0
    pack .powcontour.resbutt.high -in .powcontour.resbutt -side left \
	    -padx 4 -pady 1
    pack .powcontour.resbutt.med -in .powcontour.resbutt -side left \
	    -padx 4 -pady 1
    pack .powcontour.resbutt.low -in .powcontour.resbutt -side left \
	    -padx 4 -pady 1

    checkbutton .powcontour.separate -bg $powbg \
	    -text "Place contours in separate graph" \
	    -variable powContourParam(separate) -onvalue yes -offvalue no \
	    -highlightthickness 0 -takefocus 0

    frame .powcontour.buttons -bg $powbg
    button .powcontour.buttons.make -text "Make Contours" -bg $powbg \
	    -command { 
	powMakeContours $powContourParam(image) \
                        [.powcontour.cntrs.lst get 1.0 end] \
	                $powContourParam(res)
    }
    button .powcontour.buttons.exit -text "Exit" -bg $powbg \
	    -command {destroy .powcontour}
    pack .powcontour.buttons.make -in .powcontour.buttons -side left \
	    -padx 4 -pady 3
    pack .powcontour.buttons.exit -in .powcontour.buttons -side left \
	    -padx 4 -pady 3

    grid .powcontour.help -in .powcontour -row 0 -column 2 -sticky ne
    grid .powcontour.image -in .powcontour -row 0 -column 0 -sticky e \
	    -pady 8
    grid .powcontour.currimg -in .powcontour -row 0 -column 1 -sticky ew \
	    -pady 8 -padx 6

    grid .powcontour.imgrng -in .powcontour -row 2 -column 0 -sticky e
    grid .powcontour.imgfrm -in .powcontour -row 2 -column 1 -sticky ew \
	    -columnspan 2
    grid .powcontour.pixrng -in .powcontour -row 3 -column 0 -sticky e
    grid .powcontour.pixfrm -in .powcontour -row 3 -column 1 -sticky ew \
	    -columnspan 2
    grid .powcontour.ncntrs -in .powcontour -row 4 -column 0 -sticky e
    grid .powcontour.ncntrsbutt -in .powcontour -row 4 -column 1 -sticky w \
	    -padx 4
    grid .powcontour.scale -in .powcontour -row 5 -column 0 -sticky e
    grid .powcontour.sclbutt -in .powcontour -row 5 -column 1 -sticky w
    grid .powcontour.clist -in .powcontour -row 6 -column 0 -sticky e
    grid .powcontour.cntrs -in .powcontour -row 6 -column 1 -sticky news \
	    -padx 5 -columnspan 2
    grid .powcontour.res -in .powcontour -row 8 -column 0 -sticky e
    grid .powcontour.resbutt -in .powcontour -row 8 -column 1 -sticky w \
	    -pady 8
    grid .powcontour.separate -in .powcontour -row 10 -column 0 -sticky ew \
	    -padx 6 -pady 8 -columnspan 3

    grid .powcontour.buttons -in .powcontour -row 11 -column 0 -columnspan 3 \
	    -pady 8

    grid columnconfigure .powcontour 1 -weight 1
    grid rowconfigure .powcontour 0 -weight 1
    grid rowconfigure .powcontour 1 -minsize 10
    grid rowconfigure .powcontour 7 -minsize 10
    grid rowconfigure .powcontour 6 -weight 1
    grid rowconfigure .powcontour 11 -weight 1

    powSetContours 0 0 0
}

proc powGetScale { min max scale nlvls } {

    set offset 0.0
    if { $min<0.0 } {
	set offset [expr -2.0*$min]
	set min [expr $min+$offset]
	set max [expr $max+$offset]
    }
    set min [expr double($min)]
    set max [expr double($max)]
    set list ""

    switch $scale {
	linear {
	    set step [expr ($max-$min) / ($nlvls-1) ]
	    set val $min
	    for {set i 0} {$i<$nlvls} {incr i} {
		lappend list [expr $val-$offset]
		set val [expr $val + $step]
	    }
	}
	sqrt {
	    set step [expr ( sqrt($max) - sqrt($min) ) / ($nlvls-1) ]
	    set val [expr sqrt($min)]
	    for {set i 0} {$i<$nlvls} {incr i} {
		lappend list [expr $val*$val-$offset]
		set val [expr $val + $step]
	    }
	}
	log {
	    if {$min==0.0} {set min [expr 0.001*$max]}
	    set step [expr log( $max / $min ) / ($nlvls-1) ]
	    set val [expr log($min)]
	    for {set i 0} {$i<$nlvls} {incr i} {
		lappend list [expr exp($val)-$offset]
		set val [expr $val + $step]
	    }
	}
	exp {}
    }
    return $list
}

proc powSetContours { a b c } {
    global powContourParam

    set nContrs $powContourParam(nContrs)
    if { $nContrs == "" } return
    if { !($nContrs > 2) } { set nContrs 2 }
    set powContourParam(list) ""

    set powContourParam(list) \
	    [powGetScale $powContourParam(min) $powContourParam(max) \
                         $powContourParam(scale) $nContrs]

    if { [winfo exists .powcontour.cntrs.lst] } {
	.powcontour.cntrs.lst delete 1.0 end
	.powcontour.cntrs.lst insert end [join $powContourParam(list) "\n"]
    }
}

proc powMakeContours { img list res } {
    global powRBmin powRBmax powContourParam powPlotParam
    global powWCS powCurveParam

    set gn $powContourParam(gn)
    set cntr ${img}_contour

    if { [info exists powWCS($img)] && $powWCS($img)!="" } {
	set powWCS($cntr) $powWCS($img)
    }
    powCreateContour $cntr $img $list $res

    if { $powContourParam(separate) == "yes" } {
	set graph $cntr
	set images NULL
	set curves $cntr
    } else {
	set graph $gn
	set images $powPlotParam(images,$gn)
	set curves $powPlotParam(curves,$gn)
	if {$curves=="NULL"} {
	    set curves $cntr
	} else {
	    lappend curves $cntr
	}
    }

    # Find the true width of the of the graph box
    set width  [tagXdim .pow.pow ${gn}box]
    set height [tagYdim .pow.pow ${gn}box]

    if { [lsearch -exact [powListGraphs] $graph]>=0 } {
	powUnmapGraph $graph
    }

    set powCurveParam(lStyle${cntr},$graph) " "
    set powCurveParam(lDisp${cntr},$graph) 1
    set powCurveParam(pDisp${cntr},$graph) 0

    set fixed $powPlotParam(FixedAspect,$gn)

    powCreateGraph $graph $curves $images \
	    $powPlotParam(xunits,$gn) $powPlotParam(yunits,$gn) \
	    $powPlotParam(xlabel,$gn) $powPlotParam(ylabel,$gn) \
	    $width $height \
	    $powPlotParam(xBot,$gn) $powPlotParam(yBot,$gn) \
	    $powPlotParam(xTop,$gn) $powPlotParam(yTop,$gn)

    set powPlotParam(FixedAspect,$graph) $fixed
}

proc powMovieLoadInfo { } {
    global movieParam currgn

    set frames [.pow.pow find withtag disp$currgn]
    set nframes [llength $frames]
    set lasttags [.pow.pow gettags [lindex $frames end] ]
    regexp "frame(\[0-9\]+)" "$lasttags" tag fnum

    set movieParam(gn) $currgn
    set movieParam(nframes) $nframes
    set movieParam(title) "$currgn: $fnum of $nframes frames"
}

proc powMovie { } {
    global powPlotParam movieParam powbg currgn

    if {[winfo exists .powmovie]} {destroy .powmovie}
    powToplevel .powmovie .pow "-bg $powbg"
    catch {wm title .powmovie "Blink Images"}

    powMovieLoadInfo

    set movieParam(playing) 0
    set movieParam(speed)  10
    set movieParam(all)     1
    set movieParam(loop)    0
    set movieParam(direction) 1
    set movieParam(list)   ""

    label .powmovie.title -textvariable movieParam(title) \
	    -bg yellow -fg black
    frame .powmovie.step -bg $powbg
    button .powmovie.step.forward  -text "Next" -command {powMovieNext 1} \
	    -bg $powbg
    button .powmovie.step.addtolist -text "Add to list" \
	    -command {powMovieAddtoList} -bg $powbg
    button .powmovie.step.backward -text "Prev" -command {powMovieNext -1} \
	    -bg $powbg
    button .powmovie.step.help -text "Help" \
	    -command {powHelp Blinking.html} \
	    -bg $powbg
    pack .powmovie.step.addtolist -in .powmovie.step -side left -padx 7
    pack .powmovie.step.backward  -in .powmovie.step -side left
    pack .powmovie.step.forward   -in .powmovie.step -side left
    pack .powmovie.step.help      -in .powmovie.step -side left -padx 12

    label .powmovie.play -text "Play:" -bg $powbg -fg black
    radiobutton .powmovie.all -text "All Frames" -variable movieParam(all) \
	    -value 1 -bg $powbg -highlightthickness 0
    frame .powmovie.list -bg $powbg
    radiobutton .powmovie.list.lab -text "List " -variable movieParam(all) \
	    -value 0 -bg $powbg -highlightthickness 0
    entry .powmovie.list.txt -width 15 -bg $powbg \
	    -textvariable movieParam(list)
    pack .powmovie.list.lab -in .powmovie.list -side left
    pack .powmovie.list.txt -in .powmovie.list -side left \
	    -expand 1 -fill x

    label .powmovie.speed -text "Speed:" -bg $powbg -fg black
    frame .powmovie.speedslide -bg $powbg
    label .powmovie.speedslide.fast -text "Fast" -bg $powbg
    label .powmovie.speedslide.slow -text "Slow" -bg $powbg
    scale .powmovie.speedslide.slide -from 0 -to 20 \
	    -orient horizontal -variable movieParam(speed) \
	    -highlightbackground $powbg -bg $powbg \
	    -showvalue 0
    pack .powmovie.speedslide.fast -in .powmovie.speedslide -side left
    pack .powmovie.speedslide.slide -in .powmovie.speedslide -side left \
	    -expand 1 -fill x
    pack .powmovie.speedslide.slow -in .powmovie.speedslide -side right

    checkbutton .powmovie.loop -variable movieParam(loop) -text Loop \
	    -bg $powbg -highlightthickness 0
    checkbutton .powmovie.reverse -variable movieParam(direction) \
	    -text Reverse -bg $powbg -highlightthickness 0 \
	    -offvalue 1 -onvalue -1

    frame .powmovie.buttons -bg $powbg
    button .powmovie.buttons.reload -text "Reload Info" -bg $powbg \
	    -command {powMovieLoadInfo}
    button .powmovie.buttons.play -text "Blink Images" -bg $powbg \
	    -command {powPlayMovie} -width 12
    button .powmovie.buttons.exit -text "Exit" -bg $powbg \
	    -command {set movieParam(playing) 0; destroy .powmovie}
    pack .powmovie.buttons.reload -in .powmovie.buttons -side left \
	    -padx 4 -pady 3 
    pack .powmovie.buttons.play -in .powmovie.buttons -side left \
	    -padx 4 -pady 3
    pack .powmovie.buttons.exit -in .powmovie.buttons -side left \
	    -padx 4 -pady 3
    
    grid .powmovie.title -in .powmovie -row 0 -column 0 -columnspan 3
    grid .powmovie.step -in .powmovie -row 1 -column 0 -columnspan 3 \
	    -pady 5

    grid .powmovie.play -in .powmovie -row 2 -column 0 -sticky e
    grid .powmovie.all -in .powmovie -row 2 -column 1 -sticky w
    grid .powmovie.loop -in .powmovie -row 2 -column 2 -sticky w -padx 10
    grid .powmovie.list -in .powmovie -row 3 -column 1 -sticky ew
    grid .powmovie.reverse -in .powmovie -row 3 -column 2 -sticky w -padx 10

    grid .powmovie.speed -in .powmovie -row 4 -column 0 -sticky e -pady 3
    grid .powmovie.speedslide -in .powmovie -row 4 -column 1 -sticky ew -pady 3

    grid .powmovie.buttons -in .powmovie -row 5 -column 0 -columnspan 3 \
	    -pady 8

    grid columnconfigure .powmovie 1 -weight 1
    grid rowconfigure .powmovie 0 -weight 1
    grid rowconfigure .powmovie 1 -weight 1
    grid rowconfigure .powmovie 5 -weight 1
}

proc powMovieNext { step } {
    global movieParam powGUI currgn

    set frames [.pow.pow find withtag disp$movieParam(gn)]
    set nframes [llength $frames]
    set top [lindex $frames end]
    regexp "frame(\[0-9\]+)" [.pow.pow gettags $top] tag fnum
    incr fnum $step
    if {$fnum > $nframes} {set fnum 1} \
	    elseif {$fnum<1} {set fnum $nframes}
    
    set next [.pow.pow find withtag frame${fnum}$movieParam(gn)]
    .pow.pow raise $next $top
    set movieParam(title) "$movieParam(gn): $fnum of $nframes frames"
    regexp "(\[^ \]+)disp$movieParam(gn)" [.pow.pow gettags $next] tag img

    if {$movieParam(gn) == $currgn} {
	powSelectImage $movieParam(gn) $img

	if $powGUI {
	    set top [lindex \
		    [.pow.scope find withtag disp$movieParam(gn)scope] end]
	    set tag "frame${fnum}$movieParam(gn)scope"
	    set id [.pow.scope find withtag $tag]
	    .pow.scope raise $id $top
	}
    }
}

proc powMovieAddtoList { } {
    global movieParam

    set frames [.pow.pow find withtag disp$movieParam(gn)]
    set top [lindex $frames end]
    regexp "frame(\[0-9\]+)" [.pow.pow gettags $top] tag fnum
    if {$movieParam(list)==""} {set movieParam(list) $fnum} \
	    else {set movieParam(list) "$movieParam(list),$fnum"}
    set movieParam(all) 0
}

proc powPlayMovie { } {
    global movieParam

    set gn $movieParam(gn)
    set frames [.pow.pow find withtag disp$gn]
    set top [lindex $frames end]
    set nframes [llength $frames]

    set movieParam(playing) 1
    if { [winfo exists .powmovie] } {
	.powmovie.buttons.play configure -text "Stop Movie" \
		-command {set movieParam(playing) 0}
    }
    
    set expdlist ""
    if {$movieParam(all)} {
	for {set i 1} {$i<=$nframes} {incr i} { lappend expdlist $i }
    } else {
	set list [split $movieParam(list) " ,"]
	while { [llength $list] } {
	    set elem [lindex $list 0]
	    set list [lreplace $list 0 0]
	    if {$elem==""} continue
	    if { [regexp -- - $elem] } {
		set elem [split $elem "-"]
		set a1 [lindex $elem 0]
		set a2 [lindex $elem 1]
		if { $a1 <= $a2 } {
		    for {set i $a1} {$i<=$a2} {incr i} \
			    {lappend expdlist $i}
		} else {
		    for {set i $a1} {$i>=$a2} {incr i -1} \
			    {lappend expdlist $i}
		}
	    } elseif { [expr $elem] > 0 && [expr $elem] <= $nframes} {
		lappend expdlist [expr $elem]
	    }
	}
    }    

    set ndisp [llength $expdlist]
    if {$movieParam(direction)>0} { set Fidx 0 } else { set Fidx [expr $ndisp-1] }

    while {$movieParam(playing)} {
	set nextframe [lindex $expdlist $Fidx]
	if {$nextframe<=$nframes && $nextframe>0} {
	    set next [.pow.pow find withtag frame${nextframe}$gn]
	    .pow.pow raise $next $top
	    set top $next
	    regexp "(\[^ \]+)disp$gn" \
		    [.pow.pow gettags $next] tag img
	    powSelectImage $movieParam(gn) $img
	    set movieParam(title) \
		    "$movieParam(gn): $nextframe of $movieParam(nframes) frames"
	    set speed 1.5
	    for {set j 0} {$j<$movieParam(speed)} {incr j} {
		set speed [expr $speed*1.5]
	    }
	    set speed [expr int($speed)]
	    for {set j 0} {$j<$speed && $movieParam(playing)} {incr j} \
		    { update }
	}
	incr Fidx $movieParam(direction)
	if {$Fidx<0} {
	    set Fidx [expr $ndisp-1]
	    if {!$movieParam(loop)} { set movieParam(playing) 0 }
	} elseif {$Fidx>$ndisp} {
	    set Fidx 0
	    if {!$movieParam(loop)} { set movieParam(playing) 0 }
	}
    }

# Check that dialog box is still around before trying to change
# one of its objects

    if {[winfo exists .powmovie.buttons.play]} {
	.powmovie.buttons.play configure -text "Blink Images" \
		-command {powPlayMovie}
    }

    set top [lindex [.pow.scope find withtag disp$movieParam(gn)scope] end]
    set id [.pow.scope find withtag frame${nextframe}$movieParam(gn)scope]
    .pow.scope raise $id $top
}

proc powBlinkGraphDlg { } {
    global powPlotParam powbg currgn movieParam

    if {[winfo exists .powblink]} {destroy .powblink}
    powToplevel .powblink .pow "-bg $powbg"
    catch {wm title .powblink "Blink Graphs"}

    set movieParam(playing) 0

    listbox .powblink.listall -width 20 -height 10 \
	    -selectmode extended -bg $powbg -exportselection 0
    label .powblink.titleall -text "Available Graphs:" -fg black -bg yellow

    listbox .powblink.listblnk -width 20 -height 10 \
	    -selectmode extended -bg $powbg -exportselection 0
    label .powblink.titleblnk -text "Blink Order:" -fg black -bg yellow

    button .powblink.help -text "Help" \
	    -command {powHelp Blinking.html} \
	    -bg $powbg

    grid .powblink.titleall -in .powblink -row 1 -column 1 -sticky w \
	    -padx 5 -pady 5
    grid .powblink.listall  -in .powblink -row 2 -column 1 -sticky news
    grid .powblink.help     -in .powblink -row 1 -column 2
    grid .powblink.titleblnk -in .powblink -row 1 -column 3 -sticky w \
	    -padx 5 -pady 5
    grid .powblink.listblnk  -in .powblink -row 2 -column 3 -sticky news

    frame .powblink.grphbtns -bg $powbg
    button .powblink.grphbtns.add -text "Add -->" -bg $powbg \
	    -command {powBlinkAdd}
    button .powblink.grphbtns.insert -text "Insert -->" -bg $powbg \
	    -command {powBlinkInsert}
    button .powblink.grphbtns.delete -text "Delete <--" -bg $powbg \
	    -command {powBlinkDelete}

    frame .powblink.grphbtns.shft -bg $powbg
    label .powblink.grphbtns.shft.lab -text "Shift\n& Align\nGraphs" -bg $powbg
    canvas .powblink.grphbtns.shft.but -width 56 -height 56 -bg $powbg \
	    -highlightthickness 0
    .powblink.grphbtns.shft.but create polygon  3 30 18 22 18 38 \
	    -fill grey -outline black -tag "blinkShftLeft btn"
    .powblink.grphbtns.shft.but create polygon 30  3 22 18 38 18 \
	    -fill grey -outline black -tag "blinkShftUp btn"
    .powblink.grphbtns.shft.but create polygon 57 30 42 22 42 38 \
	    -fill grey -outline black -tag "blinkShftRght btn"
    .powblink.grphbtns.shft.but create polygon 30 57 22 42 38 42 \
	    -fill grey -outline black -tag "blinkShftDown btn"
    .powblink.grphbtns.shft.but create rectangle 23 23 37 37 \
	    -fill grey -outline black -tag "blinkShftCntr btn"
    pack .powblink.grphbtns.shft.lab -in .powblink.grphbtns.shft \
	    -side left -padx 5
    pack .powblink.grphbtns.shft.but -in .powblink.grphbtns.shft \
	    -side left -padx 5

    bind .powblink.grphbtns.shft.but <ButtonPress> {powBlinkShift 10}
    bind .powblink.grphbtns.shft.but <Shift-ButtonPress> {powBlinkShift 1}
    bind .powblink.grphbtns.shft.but <ButtonRelease> {set powShiftLoop 0}
    bind .powblink.grphbtns.shft.but <Motion> {powBlinkShiftHilight %x %y}
    bind .powblink.grphbtns.shft.but <Leave> {powBlinkShiftHilight %x %y}

    pack .powblink.grphbtns.add -in .powblink.grphbtns -fill x \
	    -padx 5 -pady 2
    pack .powblink.grphbtns.insert -in .powblink.grphbtns -fill x \
	    -padx 5 -pady 2
    pack .powblink.grphbtns.shft  -in .powblink.grphbtns \
	    -fill none -padx 5 -pady 2
    pack .powblink.grphbtns.delete -in .powblink.grphbtns -fill x \
	    -padx 5 -pady 2
    grid .powblink.grphbtns -in .powblink -row 2 -column 2

    set movieParam(speed) 10
    label .powblink.speedfast -text "Speed: Fast" -bg $powbg
    label .powblink.speedslow -text "Slow" -bg $powbg
    scale .powblink.speedslide -from 0 -to 20 \
	    -orient horizontal -variable movieParam(speed) \
	    -highlightbackground $powbg -bg $powbg \
	    -showvalue 0 
    grid .powblink.speedfast -in .powblink -row 3 -column 1 -sticky e -pady 5
    grid .powblink.speedslide -in .powblink -row 3 -column 2 -sticky ew -pady 5
    grid .powblink.speedslow -in .powblink -row 3 -column 3 -sticky w -pady 5

    frame .powblink.blnkbtns -bg $powbg
    button .powblink.blink -text "Blink Graphs" -bg $powbg \
	    -command {powBlinkBlink}
    button .powblink.reload -text "Reload Info" -bg $powbg \
	    -command {powBlinkReload}
    button .powblink.exit -text "Exit"  -bg $powbg \
	    -command {set movieParam(playing) 0; destroy .powblink}

    grid .powblink.reload -in .powblink -row 4 -column 1 -sticky e \
	    -pady 7 -padx 10
    grid .powblink.blink -in .powblink -row 4 -column 2 -sticky ew \
	    -pady 7
    grid .powblink.exit -in .powblink -row 4 -column 3 -sticky w \
	    -pady 7 -padx 10

    grid columnconfigure .powblink 1 -weight 1
    grid columnconfigure .powblink 3 -weight 1
    grid rowconfigure .powblink 2 -weight 1

    powBlinkReload
}

proc powBlinkReload { } {
    global currgn

    set graphlist ""
    foreach graph [powListGraphs] {
	if { ![regexp scope\$ $graph] } {lappend graphlist $graph}
    }
    
    .powblink.listall delete 0 end
    eval [concat .powblink.listall insert end $graphlist]
    .powblink.listall selection clear 0 end
    set curridx [lsearch -exact $graphlist $currgn]
    if {$curridx>=0} {.powblink.listall selection set $curridx} \
    else {.powblink.listall selection set 0}
}

proc powBlinkBlink { } {
    global movieParam currgn

    .powblink.blink configure -text "Stop Blinking" \
	    -command {set movieParam(playing) 0}

    set movieParam(playing) 1
    while {$movieParam(playing)} {
	if { ![winfo exists .powblink.listblnk] } {
	    set movieParam(playing) 0
	    break
	}
	set graphlist [.powblink.listblnk get 0 end]
	set cnt [llength $graphlist]
	for {set i 0} {$i<$cnt && $movieParam(playing)} {incr i} {
	    if {$cnt>0} {.pow.pow raise [lindex $graphlist $i]}
	    set speed 1.5
	    for {set j 0} {$j<$movieParam(speed)} {incr j} {
		set speed [expr $speed*1.5]
	    }
	    set speed [expr int($speed)]
	    for {set j 0} {$j<$speed && $movieParam(playing)} {incr j} \
		    { update }
	}
	update
    }

# Check that dialog box is still around before trying to change
# its objects

    if {[winfo exists .powblink]} {
	.powblink.blink configure -text "Blink Graphs" \
		-command {powBlinkBlink}
    }

    .pow.pow raise $currgn
}

proc powBlinkShift { size } {
    global currgn powShiftLoop powShiftCurrBtn

    set powShiftLoop 1
    set origobj $powShiftCurrBtn
    set firstloop 1

    while {$powShiftLoop} {
	set obj $powShiftCurrBtn
	if {$obj!="" && $obj==$origobj} {
	    set tags [.powblink.grphbtns.shft.but gettags $obj]
	    regexp "blinkShft(\[^ \]+)" $tags "" direction
	    if {$direction=="Cntr"} {
		set select [.powblink.listblnk get 0 end]
	    } else {
		set select ""
		foreach idx [.powblink.listblnk curselection] {
		    lappend select [.powblink.listblnk get $idx]
		}
	    }
	    foreach graph $select {
		set dx 0
		set dy 0
		switch -exact $direction {
		    Cntr {
			if { ![info exists bbox1] } {
			    set cbbox [.pow.pow cget -scrollregion]
			    set bbox1 [.pow.pow bbox $graph]
			    set xloc [expr double( \
				    [lindex $bbox1 0]+[lindex $bbox1 2]) \
				    / [lindex $cbbox 2] / 2.0 ]
			    set yloc [expr double( \
				    [lindex $bbox1 1]+[lindex $bbox1 3]) \
				    / [lindex $cbbox 3] / 2.0 ]
			    set xv [.pow.pow xview]
			    if {$xloc<[lindex $xv 0] || $xloc>[lindex $xv 1]} {
				.pow.pow xview moveto \
					[expr double([lindex $bbox1 0]-30) \
					/ [lindex $cbbox 2] ]
			    }
			    set yv [.pow.pow yview]
			    if {$yloc<[lindex $yv 0] || $yloc>[lindex $yv 1]} {
				.pow.pow yview moveto \
					[expr double([lindex $bbox1 1]-30) \
					/ [lindex $cbbox 3] ]
			    }
			} else {
			    set bbox2 [.pow.pow bbox $graph]
			    set dx [expr [lindex $bbox1 2]-[lindex $bbox2 2]]
			    set dy [expr [lindex $bbox1 1]-[lindex $bbox2 1]]
			}
		    }
		    Up   { set dy -$size }
		    Down { set dy  $size }
		    Left { set dx -$size }
		    Rght { set dx  $size }
		}
		powMoveGraph $graph $dx $dy
	    }
	}
	if {$firstloop} {
	    for {set delay 0} {$delay<200} {incr delay} {update}
	}
	update
	set firstloop 0
    }
}

proc powBlinkShiftHilight { x y } {
    global powbg powShiftCurrBtn

    .powblink.grphbtns.shft.but itemconfigure btn -fill grey
    set obj [.powblink.grphbtns.shft.but find overlapping $x $y $x $y]
    if {[regexp btn [.powblink.grphbtns.shft.but gettags $obj] ]} {
	set powShiftCurrBtn $obj
	.powblink.grphbtns.shft.but itemconfigure $obj -fill white
    } else {
        set powShiftCurrBtn ""
    }
}


proc powBlinkAdd { } {

    set count 0
    set select [.powblink.listall curselection]
    foreach item $select {
	set additem [.powblink.listall get $item]
	.powblink.listblnk insert end $additem
	incr count
    }
    if {$count>0} {
	set nelem [llength [.powblink.listblnk get 0 end]]
	.powblink.listblnk selection clear 0 end
	.powblink.listblnk selection set [expr $nelem-$count] end
    }
}

proc powBlinkInsert { } {

    set count 0
    set select [.powblink.listall curselection]
    set insloc [lindex [.powblink.listblnk curselection] 0]
    if {$insloc==""} {set insloc 0}
    foreach item $select {
	set additem [.powblink.listall get $item]
	.powblink.listblnk insert [expr $insloc+$count] $additem
	incr count
    }
    if {$count>0} {
	.powblink.listblnk selection clear 0 end
	.powblink.listblnk selection set $insloc [expr $insloc+$count-1]
    }
}

proc powBlinkDelete { } {

    set count 0
    set select [.powblink.listblnk curselection]
    foreach item $select {
	.powblink.listblnk delete [expr $item-$count]
	incr count
    }
    if {$count>0} {
	.powblink.listblnk selection set [lindex $select 0]
    }
}


proc powAdornGraph {gn {canvas ".pow.pow"}} {
    global powfont powPlotParam powSixties
    global currimg

    if {$canvas != ".pow.pow"} {return}

    foreach el  [array names powPlotParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == $gn } {
	    set $p1 $powPlotParam($p1,$p2)
        }
    }

#strip any previous "adornments"
    .pow.pow delete ${gn}ticks ${gn}nums ${gn}label ${gn}grid


    if { $powSixties && [powWCSexists $gn] } {
	set useSixties 1
    } else {
	set useSixties 0
    }

    #  Make tick frequency non-linear
    set NumTicks [expr $NumTicks + int(exp($NumTicks/3.0))]
    set ticks [powDrawGridLines $gn $canvas $useSixties \
               $GridColor $NumTicks $GridDash $Grid]

    foreach {x y val axis side} $ticks {
	if {$axis=="x"} {
	    if {$useSixties} {
		set label [powHourRA $val]
		regsub {\..*s$} $label "s" label
	    } else {
		set label [format "%.6lg" $val ]
	    }
	} elseif {$axis=="y"} {
	    if {$useSixties} {
		set label [powDegDec $val]
		regsub {\..*$} $label "" label
	    } else {
		set label [format "%.5lg" $val ]
	    }   
	}
	
	switch $side {
	    rgt {
		$canvas create line $x $y [expr $x + 10] $y \
	-tags "$gn ${gn}line ${gn}ticks ${gn}rgtticks ${gn}${axis}ticks" \
			-fill black
	    }
	    top {
		$canvas create line $x $y $x [expr $y - 10] \
	-tags "$gn ${gn}line ${gn}ticks ${gn}topticks ${gn}${axis}ticks" \
			-fill black
	    }
	    lft {
		$canvas create line $x $y [expr $x - 10] $y \
	-tags "$gn ${gn}line ${gn}ticks ${gn}lftticks ${gn}${axis}ticks" \
			-fill black
		set id [$canvas create text [expr $x - 15] $y \
			-text $label -anchor e -font $powfont \
	-tags "$gn ${gn}text ${gn}nums ${gn}lftnums ${gn}${gn}nums"]
	    }
	    bot {
		$canvas create line $x $y $x [expr $y + 10] \
	-tags "$gn ${gn}line ${gn}ticks ${gn}botticks ${gn}${axis}ticks" \
			-fill black
		set id [$canvas create text $x [expr $y + 15] \
			-text $label -anchor n -font $powfont \
	-tags "$gn ${gn}text ${gn}nums ${gn}botnums ${gn}${axis}nums"]
	    }
	}
    }

# put the X and Y labels 
    if { [powWCSexists $gn] } {
	set xString "RA"
	set yString "Dec"
	set powPlotParam(xString,$gn) $xString
	set powPlotParam(yString,$gn) $yString
    }

    set bbox [$canvas coords ${gn}box]

    if [regexp {[^ ]} $xString] {
	.pow.pow create text [expr ([lindex $bbox 2] + [lindex $bbox 0])/2 ] \
	    [expr [lindex $bbox 3] + 35] -text $xString -anchor n \
	    -tags "$gn ${gn}label ${gn}xlabel ${gn}text" -font $powfont
    }

    if [regexp {[^ ]} $yString] {
	.pow.pow create text [expr [lindex $bbox 0] - $xmargin/2] \
	    [expr [lindex $bbox 1] - 5] -text $yString -anchor sw \
            -justify left -tags "$gn ${gn}label ${gn}ylabel ${gn}text"\
            -font $powfont
    }
}

proc powDummyRangeCallback { gn x0 x1} {
    puts "You have selected the ordered pair: ( $x0 , $x1) on the graph $gn"
}


proc powDragRange { x_or_y {tag highlight} {color red} {callback powDummyRangeCallback}} {
    global currgn powRangeX0 powRangeX1 powRangeX0C powRangeXC 
    global powRangeY0 powRangeY1 powRangeY0C powRangeYC powRangeTag powRangeColor
    global powRangeCallback  powRangeSaveBinding
    
    set powRangeCallback $callback
    set powRangeTag $tag
    set powRangeColor $color
    
    set powRangeSaveBinding(ButtonPress-1) [bind .pow.pow <ButtonPress-1>]
    set powRangeSaveBinding(B1-Motion) [bind .pow.pow <B1-Motion>]
    set powRangeSaveBinding(ButtonRelease-1) [bind .pow.pow <ButtonRelease-1>]
    
    bind .pow.pow <ButtonPress-1> {
	set gn [powWhereAmI %x %y]; 
	if {$gn == $currgn} {
	    set powRangeX0C [.pow.pow canvasx %x];
	    set powRangeY0C [.pow.pow canvasy %y];
	    set gcoords [powCanvasToGraph $currgn $powRangeX0C $powRangeY0C \
			     .pow.pow];
	    set powRangeX0 [lindex gcoords 0]
	    set powRangeY0 [lindex gcoords 1]
	    .pow.pow create line $powRangeX0C $powRangeY0C $powRangeX0C $powRangeY0C \
		-tags Range -fill $powRangeColor
	}
    }
    bind .pow.pow <B1-Motion> {
	set gn [powWhereAmI %x %y]; 
	if {$gn == $currgn} {
	    if {![info exists powRangeX0C]} {
		set powRangeX0C [.pow.pow canvasx %x];
		set powRangeY0C [.pow.pow canvasy %y];
		set gcoords [powCanvasToGraph $currgn $powRangeX0C $powRangeY0C \
			     .pow.pow];
		set powRangeX0 [lindex gcoords 0]
		set powRangeY0 [lindex gcoords 1]
	    } else {
		.pow.pow delete Range; 
	    }
	    set powRangeXC [.pow.pow canvasx %x];
	    set powRangeYC [.pow.pow canvasy %y];
	    .pow.pow create line $powRangeX0C $powRangeY0C $powRangeXC $powRangeYC \
		-tags Range -fill $powRangeColor
	}
    }
    if {$x_or_y == "X"} { 
	bind .pow.pow <ButtonRelease-1> {
	    if {[info exists powRangeX0C]} {
		set range_coords [.pow.pow coords Range]
		.pow.pow delete Range
		set powRangeXC [lindex $range_coords 0];
		set powRangeYC [lindex $range_coords 1];
		set powRangeCoords [powCanvasToGraph $currgn \
                               $powRangeXC $powRangeYC .pow.pow]
		set powRangeX0 [lindex $powRangeCoords 0]
		set powRangeY0 [lindex $powRangeCoords 1]
		set powRangeXC [lindex $range_coords 2];
		set powRangeYC [lindex $range_coords 3];
		set powRangeCoords [powCanvasToGraph $currgn \
                               $powRangeXC $powRangeYC .pow.pow]
		set powRangeX1 [lindex $powRangeCoords 0]
		set powRangeY1 [lindex $powRangeCoords 1]
		powColorRange $currgn X $powRangeX0 $powRangeX1 $powRangeY0 $powRangeY1 $powRangeTag $powRangeColor 0;
		$powRangeCallback $currgn $powRangeX0 $powRangeX1; 
		bind .pow.pow <ButtonPress-1> \
		    "$powRangeSaveBinding(ButtonPress-1)";
		bind .pow.pow <B1-Motion> \
		    "$powRangeSaveBinding(B1-Motion)";
		bind .pow.pow <ButtonRelease-1> \
		    "$powRangeSaveBinding(ButtonRelease-1)";
		#clear start point for next Drag
		unset powRangeX0C
	    }
	}
    } else {
	bind .pow.pow <ButtonRelease-1> {
	    if {[info exists powRangeX0C]} {
		set range_coords [.pow.pow coords Range]
		.pow.pow delete Range
		set powRangeXC [lindex $range_coords 0];
		set powRangeYC [lindex $range_coords 1];
		set powRangeCoords [powCanvasToGraph $currgn \
                               $powRangeXC $powRangeYC .pow.pow]
		set powRangeX0 [lindex $powRangeCoords 0]
		set powRangeY0 [lindex $powRangeCoords 1]
		set powRangeXC [lindex $range_coords 2];
		set powRangeYC [lindex $range_coords 3];
		set powRangeCoords [powCanvasToGraph $currgn \
                               $powRangeXC $powRangeYC .pow.pow]
		set powRangeX1 [lindex $powRangeCoords 0]
		set powRangeY1 [lindex $powRangeCoords 1]
		powColorRange $currgn Y $powRangeX0 $powRangeX1 $powRangeY0 $powRangeY1 $powRangeTag $powRangeColor 0;
		$powRangeCallback $currgn $powRangeY0 $powRangeY1; 
		bind .pow.pow <ButtonPress-1> \
		    "$powRangeSaveBinding(ButtonPress-1)";
		bind .pow.pow <B1-Motion> \
		    "$powRangeSaveBinding(B1-Motion)";
		bind .pow.pow <ButtonRelease-1> \
		    "$powRangeSaveBinding(ButtonRelease-1)";
		#clear start point for next Drag
		unset powRangeX0C
	    }
	}
    }
}

proc powColorRange { gn x_or_y x0 x1 y0 y1 {tag highlight} {color red} {redrawing 0}} {
    global powGraphsTagRangeList powTagsColorMap
    global chainToAxisHash axisToChainHash
#    puts "powColorRange: $gn $x_or_y $x0 $x1 $y0 $y1 $tag $color $redrawing"

    if {$x_or_y == "X"} {
	set a0 $x0
	set a1 $x1
    } else {
	set a0 $y0
	set a1 $y1
    }



    if {$a0 > $a1} {
	set tmp $a0
	set a0 $a1
	set a1 $tmp
    }
    if {!$redrawing} {
	set powTagsColorMap($tag) $color
	lappend powGraphsTagRangeList($gn) "$x_or_y $a0 $a1 $tag"
    }	

    set graphlist ${gn}$x_or_y


    if {$x_or_y =="X"} {
	if {[array names axisToChainHash ${gn}X] != ""} {
	    set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${gn}X))]
	}
    } else {
	if {[array names axisToChainHash ${gn}Y] != ""} {
	    set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${gn}Y))]
	}
    }
    

    foreach graph [concat $graphlist] {
	set axis [chopped $graph]
	set graph [chop $graph]
	powTagRange $graph  $x_or_y $x0 $x1 $y0 $y1 $tag
    }

    .pow.pow itemconfigure $tag -fill $color
}


proc powTagRange { gn x_or_y x0 x1 y0 y1 tag } {
#    puts "powTagRange: $gn $x_or_y $x0 $x1 $tag"
    set gbox [.pow.pow coords ${gn}box]
    set gx0 [lindex $gbox 0]
    set gx1 [lindex $gbox 2]
    set gy0 [lindex $gbox 1]
    set gy1 [lindex $gbox 3]
    if {$x_or_y == "X"} {
	set xa [lindex [powGraphToCanvas $gn $x0 $y0 .pow.pow] 0]
	set xb [lindex [powGraphToCanvas $gn $x1 $y1 .pow.pow] 0]
	if {($xa < $gx0 && $xb < $gx0) || ($xa > $gx1 && $xb > $gx1)} {
	    #range is entirely off the graph
	    return
	} 
	if {$xa < $gx0} {set xa $gx0 }
	if {$xb > $gx1} {set xb $gx1 }
	set ya $gy0
	set yb $gy1
    } else {
	set xa $gx0
	set xb $gx1
	set ya [lindex [powGraphToCanvas $gn $x1 $y1 .pow.pow] 1]
	set yb [lindex [powGraphToCanvas $gn $x0 $y0 .pow.pow] 1]
	if {($ya < $gy0 && $yb < $gy0) || ($ya > $gy1 && $yb > $gy1)} {
	    #range is entirely off the graph
	    return
	} 
	if {$ya < $gy0} {set ya $gy0 }
	if {$yb > $gy1} {set yb $gy1 }
    }
    .pow.pow addtag $tag enclosed $xa $ya $xb $yb
}

proc powTagRect { gn x0 y0 x1 y1 tag } {
    if {$x0 > $x1} {
	set tmp $x0
	set x0 $x1
	set x1 $tmp
    }
    if {$y0 > $y1} {
	set tmp $y0
	set y0 $y1
	set y1 $tmp
    }
    set ccoords [powGraphToCanvas $gn $x0 $y0 .pow.pow]
    set xa [lindex $ccoords 0]
    set yb [lindex $ccoords 1]
    set ccoords [powGraphToCanvas $gn $x1 $y1 .pow.pow]
    set xb [lindex $ccoords 0]
    set ya [lindex $ccoords 1]
    set gbox [.pow.pow coords ${gn}box]
    set gx0 [lindex $gbox 0]
    set gx1 [lindex $gbox 2]
    set gy0 [lindex $gbox 1]
    set gy1 [lindex $gbox 3]
    if {($xa < $gx0 && $xb < $gx0) || ($xa > $gx1 && $xb > $gx1) || \
	    ($ya < $gy0 && $yb < $gy0) || ($ya > $gy1 && $yb > $gy1)} {
#rect is entirely off the displayed graph
	return
    } 
    if {$xa < $gx0} {set xa $gx0 }
    if {$xb > $gx1} {set xb $gx1 }
    if {$ya < $gy0} {set ya $gy0 }
    if {$yb > $gy1} {set yb $gy1 }
    
    .pow.pow  addtag $tag enclosed $xa $ya $xb $yb
}





proc powDummyRectCallback { gn  x0 y0 x1 y1} {
    puts "You have selected the rectangle: ( $x0 , $y0 , $x1, $y1) on the graph "
}


proc powDragRect { {tag highlight} {color red} {callback powDummyRectCallback}} {
    global currgn powRectX0 powRectX1 powRectX0C powRectXC 
    global powRectY0 powRectY1 powRectY0C powRectYC powRectTag powRectColor
    global powRectCallback powRectSaveBinding

    set powRectCallback $callback
    set powRectTag $tag
    set powRectColor $color

    set powRectSaveBinding(ButtonPress-1) [bind .pow.pow <ButtonPress-1>]
    set powRectSaveBinding(B1-Motion) [bind .pow.pow <B1-Motion>]
    set powRectSaveBinding(ButtonRelease-1) [bind .pow.pow <ButtonRelease-1>]

    
    bind .pow.pow <ButtonPress-1> {
	set gn [powWhereAmI %x %y]; 
	if {$gn == $currgn} {
	    set powRectX0C [.pow.pow canvasx %x];
	    set powRectY0C [.pow.pow canvasy %y];
	    set gcoords [powCanvasToGraph $currgn $powRectX0C $powRectY0C .pow.pow];

	    set powRectX0 [lindex $gcoords 0]
	    set powRectY0 [lindex $gcoords 1]

	    .pow.pow create rectangle $powRectX0C $powRectY0C $powRectX0C $powRectY0C \
		-tags Rect -outline $powRectColor
	}
    }
    bind .pow.pow <B1-Motion> {
	set gn [powWhereAmI %x %y]; 
	if {$gn == $currgn} {
	    if {![info exists powRectX0C]} {
		set powRectX0C [.pow.pow canvasx %x];
		set powRectX0 [powCanvasToGraph $currgn X $powRectX0C .pow.pow];
		set powRectY0C [.pow.pow canvasy %y];
		set powRectY0 [powCanvasToGraph $currgn Y $powRectY0C .pow.pow];
	    } else {
		.pow.pow delete Rect; 
	    }
	    set powRectXC [.pow.pow canvasx %x];
	    set powRectYC [.pow.pow canvasy %y];
	    .pow.pow create rectangle $powRectX0C $powRectY0C $powRectXC $powRectYC \
		-tags Rect -outline $powRectColor
	}
    }
    bind .pow.pow <ButtonRelease-1> {
	if {[info exists powRectX0C]} {
	    set rect_coords [.pow.pow coords Rect]
	    .pow.pow delete Rect
	    set powRectXC [lindex $rect_coords 0]
	    set powRectYC [lindex $rect_coords 1];
	    set gcoords [powCanvasToGraph $currgn $powRectXC $powRectYC .pow.pow];
	    set powRectX0 [lindex $gcoords 0]
	    set powRectY0 [lindex $gcoords 1]

	    set powRectXC [lindex $rect_coords 2]
	    set powRectYC [lindex $rect_coords 3];
	    set gcoords [powCanvasToGraph $currgn $powRectXC $powRectYC .pow.pow]
	    set powRectX1 [lindex $gcoords 0]
	    set powRectY1 [lindex $gcoords 1]
	    powColorRect $currgn $powRectX0 $powRectY0 $powRectX1 $powRectY1 $powRectTag $powRectColor 0;
	    bind .pow.pow <ButtonPress-1> \
		"$powRectSaveBinding(ButtonPress-1)";
	    bind .pow.pow <B1-Motion> \
		"$powRectSaveBinding(B1-Motion)";
	    bind .pow.pow <ButtonRelease-1> \
		"$powRectSaveBinding(ButtonRelease-1)";
	    $powRectCallback $currgn $powRectX0 $powRectY0 $powRectX1 $powRectY1;
	}
    }
}

proc powColorRect { gn  x0 y0 x1 y1 {tag highlight} {color red} {redrawing 0}} {
    global powGraphsTagRectList powTagsColorMap
#    puts "powColorRect: $gn $x_or_y $x0 $x1 $tag $color $redrawing"
    if {$x0 > $x1} {
	set tmp $x0
	set x0 $x1
	set x1 $tmp
    }
    if {!$redrawing} {
	set powTagsColorMap($tag) $color
	lappend powGraphsTagRectList($gn) "$x0 $y0 $x1 $y1 $tag"
    }	
    powTagRect $gn $x0 $y0 $x1 $y1 $tag

# Unless someone can come up with a good reason, powColorRect doesn't follow
# linked axes because it's unclear what to do with the unlinked axis
#    set graphlist $gn

#    if {[array names axisToChainHash ${gn}X] != ""} {
#	set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${gn}X))]
#    }
#
#    if {[array names axisToChainHash ${gn}Y] != ""} {
#	set graphlist [concat $graphlist $chainToAxisHash($axisToChainHash(${gn}Y))]
#    }
    
#    foreach graph [concat $graphlist] {
#	set axis [chopped $graph]
#	set graph [chop $graph]
#	powTagRange $graph  $x0 $y0 $x1 $y1 $tag
	
#    }
    .pow.pow itemconfigure $tag -fill $color
}

proc powColorbar { } {
    global currimg currgn curr_img powResizeMain powRBmin powRBmax
    global powGrid powPlotParam powCurrentLut

    set saveimg $currimg
    set savegn $currgn

    if {[.pow.pow find withtag ${currimg}disp${currgn}] == ""} {
	puts "Your selected image must be on the selected graph to make a colorbar"
	return
    }

    set colorbar ${currgn}_colorbar

    if { [lsearch [powListGraphs] $colorbar] != -1 } {
	powUnmapGraph $colorbar
    }

    set width 50.0
    set min $powRBmin($curr_img)
    set max $powRBmax($curr_img)
    set increment [expr ($max - $min) / ($width-1)]

    set x $min
    for {set j 0} {$j < $width} {incr j} {
	lappend color_list $x
	set x [expr $x + $increment]
    }

    powCreateDataFromList $colorbar $color_list
    
    set zunits [powGetImageUnits $currimg Z]

    powCreateImage $colorbar $colorbar 0 0 \
	    [expr int($width)] 1 $min $increment 0.5 1.0 \
	    $zunits " " $zunits

    
    set powPlotParam(handletext,$colorbar) "$currgn Colorbar"
    set powPlotParam(handleposition,$colorbar) "bl"
    set powPlotParam(handleanchor,$colorbar) "nw"
    set powPlotParam(Grid,$colorbar) 0
    set powPlotParam(Colorbar,$colorbar) $curr_img
    set powPlotParam(FixedAspect,$colorbar) "no"
    set powPlotParam(FixedAspect,${colorbar}scope) "no"

    powCreateGraph $colorbar NULL $colorbar $zunits NULL " " " " \
	    [tagXdim .pow.pow ${currgn}box] 30

#remove y axis ticks and numbers
#Note currgn is now the colorbar graph
    .pow.pow delete ${currgn}yticks
    .pow.pow delete ${currgn}lftnums
    powMakeGraphHandles $currgn
    
#move colorbar to sit below original graph

    set newx [lindex [.pow.pow bbox ${savegn}box] 0]
    set newy [lindex [.pow.pow bbox $savegn] 3] 
    
    powMoveGraphTo $currgn  $newx [expr $newy + 20]

    powReconfigureToplevel $powResizeMain


#reselect original image    
    powSelectGraph $savegn
    powSelectImage $savegn $saveimg

 
}
    
# convert a decimal degree to HH MM SS.S
proc powHourRA {deciValue} {
#Written by J. Xu
    if { $deciValue < 0} {
	set deciValue [expr $deciValue + 360]
    }
    set hourValue [expr $deciValue/15.0 + 1e-13]
    set hour [expr int($hourValue)]
    set minuValue [expr ($hourValue - $hour)*60.0]
    set minu [expr int($minuValue)]
    set scndValue [expr ($minuValue - $minu)*60.0]
    set scnd $scndValue
    while {$hour >= 24} {set hour [expr $hour - 24]}
    while {$hour < 0} {set hour [expr $hour + 24]}
#save us from rounding
    if {$scnd >= 59.995} {
	set scnd 0
	incr minu
	if {$minu == 60} { set minu 0; incr hour; }
    }
    return [format "%dh%02dm%05.2fs" $hour $minu $scnd]

}

# convert a decimal degree to DD MM SS.S
proc powDegDec {deciValue} {
#Written by J. Xu
    if { $deciValue < 0} {
	set isNeg 1
    } else {
	set isNeg 0
    }
    set deciValue [expr abs($deciValue) + 1e-13]
    set deg [expr int($deciValue)]
    while {$deg > 360} {set deg [expr $deg - 360]}
    while {$deg < -360} {set deg [expr $deg + 360]}
    set minuValue [expr ($deciValue - $deg)*60.0]
    set minu [expr int($minuValue)]
    set scndValue [expr ($minuValue - $minu)*60.0]
    set scnd $scndValue
#save us from rounding
    if {$scnd >= 59.995} {
	set scnd 0
	incr minu
	if {$minu == 60} { set minu 0; incr deg; }
    }
    if { $isNeg == 1} {
	return [format "-%d:%02d:%05.2f" $deg $minu $scnd]
    } else {
	return [format "%d:%02d:%05.2f" $deg $minu $scnd]
    }
}

